"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import subprocess
import json
import numpy as np
import datetime
import re
import shutil
import os
import glob
from PyQt5.QtWidgets import QVBoxLayout, QLabel, QPushButton, QWidget, QLineEdit, QHBoxLayout, QComboBox, QScrollArea, QSpacerItem, QWidgetItem, QCheckBox
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import pyqtSlot, Qt, QSize, pyqtSignal

from . import transphire_utils as tu


class SelectDialog(QWidget):
    """
    Show a message box with an input field.

    Inherits:
    QDialog

    Signals:
    None
    """
    sig_start = pyqtSignal(object)
    sig_new_config = pyqtSignal(str, str)

    def __init__(self, *args, parent=None, **kwargs):
        """
        Initialise layout of the widget.

        Arguments:
        regex - Regularexpression to analyse
        parent - Parent widget

        Returns:
        None
        """
        super(SelectDialog, self).__init__(parent)
        self.columns = 10
        self.settings = None
        self.content = []
        self.widgets_dict = {}
        self.widgets = []
        self.neutral_name = 'neutral'
        self.bad_name = 'bad'
        self.good_name = 'good'
        self.input_name = 'input_files'
        self.model_name = 'model.h5'
        self.config_name = 'config.json'
        self.default_project_name = 'Project'
        self.model_out = None
        self.config_out = None
        self.current_index = 0
        self.layouts = {}
        self.labels = {}
        self.button_dict = {}

        self.current_model = None

        self.log_folder = None
        self.good_folder = None
        self.bad_folder = None
        self.classes_folder = None
        self.settings_file = None
        self.e2proc2d_exec = None
        self.sp_cinderella_train_exec = None
        self.sp_cinderella_predict_exec = None
        self.set_enable = None

        layout = QVBoxLayout(self)
        layout_h0 = QHBoxLayout()
        layout.addLayout(layout_h0)
        layout_h1 = QHBoxLayout()
        layout.addLayout(layout_h1)
        layout_h2 = QHBoxLayout()
        layout.addLayout(layout_h2)
        layout_h3 = QHBoxLayout()
        layout.addLayout(layout_h3)

        btn_done = QPushButton('Retrain', self)
        btn_done.clicked.connect(self.retrain)
        layout_h3.addWidget(btn_done)
        self.content.append(btn_done)

        layout_h3.addStretch(1)

        label_threshold = QLabel('Threshold', self)
        layout_h3.addWidget(label_threshold)
        self.content.append(label_threshold)

        self.threshold_repick = QLineEdit('0.5', self)
        layout_h3.addWidget(self.threshold_repick)
        self.content.append(self.threshold_repick)

        self.button_repick = QPushButton('Repick', self)
        self.button_repick.clicked.connect(self.repick)
        layout_h3.addWidget(self.button_repick)
        self.content.append(self.button_repick)

        btn = QPushButton('Set', self)
        btn.clicked.connect(self.set_settings)
        layout_h3.addWidget(btn)
        self.content.append(btn)

        self.edit = QLineEdit(str(self.columns), self)
        self.edit.editingFinished.connect(self.adjust_all_layouts)
        layout_h0.addWidget(QLabel('Columns:', self))
        layout_h0.addWidget(self.edit)
        self.content.append(self.edit)

        self.combo_text = QComboBox(self)
        self.combo_text.setSizeAdjustPolicy(QComboBox.AdjustToContents)
        self.combo_text.currentTextChanged.connect(self.set_current_folder)
        layout_h0.addWidget(self.combo_text)
        self.content.append(self.combo_text)

        self.btn_update = QPushButton('Update', self)
        self.btn_update.clicked.connect(self.start_retrain)
        layout_h0.addWidget(self.btn_update)
        self.content.append(self.btn_update)

        self.prefer_isac_checkbox = QCheckBox('Prefer ISAC', self)
        self.prefer_isac_checkbox.setToolTip('If Project is selected, prefer ISAC over Cinderella runs. This will put all classes in the neutral position')
        layout_h0.addWidget(self.prefer_isac_checkbox)
        self.content.append(self.prefer_isac_checkbox)

        for current_name in (self.good_name, self.neutral_name, self.bad_name):
            self.labels[current_name] = QLabel(current_name, self)
            self.widgets_dict[current_name] = []
            layout_h1.addWidget(self.labels[current_name])

            widget = QWidget(self)
            scroll_area = QScrollArea(self)
            scroll_area.setWidgetResizable(True)
            scroll_area.setWidget(widget)
            scroll_area.setHorizontalScrollBarPolicy( Qt.ScrollBarAlwaysOn )
            scroll_area.setVerticalScrollBarPolicy( Qt.ScrollBarAlwaysOn)

            self.layouts[current_name] = QVBoxLayout(widget)
            self.layouts[current_name].setSpacing(2)
            self.layouts[current_name].addStretch(1)
            layout_h2.addWidget(scroll_area)

        self.sig_start.connect(self.start_retrain)
        self.adjust_all_layouts()
        self.enable(False)

    @pyqtSlot()
    def set_settings(self):
        try:
            name, threshold = self.combo_text.currentText().rsplit('_', 1)
        except ValueError:
            tu.message('Please provide a repicking run and not {}.'.format(self.default_project_name))
            return
        model = os.path.join(self.log_folder, name, self.model_name)
        self.sig_new_config.emit(model, threshold)
        tu.message('Set model: {}\nSet threshold: {}'.format(model, threshold))

    def enable(self, var, use_all=None):
        if not self.set_enable and var and self.set_enable is not None:
            var = False
        for entry in self.content:
            entry.setEnabled(var)

        if var:
            var = False
            if self.combo_text.currentText() == self.default_project_name:
                var = True
            self.btn_update.setEnabled(var)
            self.prefer_isac_checkbox.setEnabled(var)

    @pyqtSlot(str)
    @pyqtSlot()
    def set_current_folder(self, text=None):
        if text is None:
            text = self.combo_text.currentText()

        var = False
        if text == self.default_project_name:
            var = True
        self.btn_update.setEnabled(var)
        self.prefer_isac_checkbox.setEnabled(var)
        self.start_retrain(input_folder=text)

    def add_combo_item(self, items):
        current_items = [self.combo_text.itemText(idx) for idx in range(self.combo_text.count())]
        current_items.extend(items)
        prev_state = self.combo_text.blockSignals(True)
        self.combo_text.clear()
        self.combo_text.addItems(sorted(list(set(current_items))))
        self.combo_text.blockSignals(prev_state)

    @pyqtSlot(object)
    @pyqtSlot()
    def start_retrain(self, settings=None, input_folder=None):
        if settings is not None:
            self.settings = settings
            file_names = [
                os.path.basename(entry)
                for entry in
                glob.glob(os.path.join(self.settings['log_folder'], 'Retrain', 'RUN_*.*'))
                ]
            self.add_combo_item([self.default_project_name])
            self.add_combo_item(file_names)
        self.clear()
        self.project_folder = self.settings['project_folder']
        self.log_folder = os.path.join(self.settings['log_folder'], 'Retrain')
        self.classes_folder = os.path.join(self.log_folder, 'RUN_{0}')
        self.model_out = os.path.join(self.classes_folder, self.model_name)
        self.config_out = os.path.join(self.classes_folder, self.config_name)

        self.good_folder = os.path.join(self.classes_folder, self.good_name)
        self.bad_folder = os.path.join(self.classes_folder, self.bad_name)

        self.settings_file = os.path.join(self.log_folder, 'tmp_settings.json')
        self.e2proc2d_exec = self.settings['Path']['e2proc2d.py']
        self.sp_cinderella_train_exec = self.settings['Path']['sp_cinderella_train.py']
        try:
            self.sp_cinderella_predict_exec = self.settings['Path'][self.settings['Copy']['Select2d']]
        except KeyError:
            print('In order to use the retrain tool, please provide a program in Copy->Select2d and press: Monitor Start -> Monitor Stop')
            self.set_enable = False
            return
        else:
            self.set_enable = True

        for folder_name in (
                self.log_folder,
                ):
            tu.mkdir_p(folder_name)

        class_2d_folder = []
        select_2d_folder = []
        if input_folder is None or input_folder == self.default_project_name:
            for key in self.settings:
                if key.startswith('scratch_'):
                    continue
                if 'class2d' in key.lower():
                    try:
                        class_2d_folder.extend([
                            entry
                            for entry in
                            glob.glob(
                                os.path.join(
                                    self.settings[key],
                                    '*',
                                    )
                                )
                            if os.path.isdir(entry) and not
                            os.path.basename(entry).startswith('jpg') and not
                            os.path.basename(entry).startswith('overview_plots')
                            ])
                    except IndexError:
                        pass
                elif 'select2d' in key.lower():
                    try:
                        select_2d_folder.extend([
                            entry
                            for entry in
                            glob.glob(
                                os.path.join(
                                    self.settings[key],
                                    '*',
                                    )
                                )
                            if os.path.isdir(entry) and not
                            os.path.basename(entry).startswith('jpg') and not
                            os.path.basename(entry).startswith('overview_plots')
                            ])
                    except IndexError:
                        pass

            if self.prefer_isac_checkbox.isChecked():
                select_basenames = tuple([os.path.basename(entry) for entry in class_2d_folder])
                for entry in select_2d_folder[:]:
                    if os.path.basename(entry) in select_basenames:
                        select_2d_folder.remove(entry)
            else:
                select_basenames = tuple([os.path.basename(entry) for entry in select_2d_folder])
                for entry in class_2d_folder[:]:
                    if os.path.basename(entry) in select_basenames:
                        class_2d_folder.remove(entry)
        else:
            select_2d_folder.append(os.path.join(self.log_folder, input_folder))

        self.fill(class_2d_folder)
        self.fill(select_2d_folder, cinderella=True)
        for label_name in self.labels:
            for idx in reversed(range(self.current_index, len(self.widgets))):
                button = self.widgets[idx]
                self.widgets.remove(button)
                button.setParent(None)
        self.adjust_all_layouts()

    def fill(self, folder, cinderella=False):
        if cinderella:
            labels = (self.good_name,self.bad_name)
        else:
            labels = (self.neutral_name,)

        button_dict = {}
        for sub_folder in folder:
            for label_name in labels:
                suffix = '_{}'.format(label_name) if cinderella else ''
                for file_name in sorted(glob.glob(os.path.join(sub_folder, 'png{}'.format(suffix), '*'))):
                    match = re.search('/([^/]*)-(\d*)\.+png', file_name)
                    if match is None:
                        continue
                    class_id = int(match.group(2))-1
                    class_averages = os.path.join(sub_folder, '' if cinderella else 'ISAC2', match.group(1))
                    try:
                        button = self.widgets[self.current_index]
                        button.isac_class_averages = class_averages
                        button.isac_class_id = class_id
                        button.current_layout = label_name
                    except KeyError:
                        button = MyPushButton(label_name, class_averages, class_id, self)
                        button.sig_click.connect(self.handle_change)
                        self.widgets.append(button)
                    except IndexError:
                        button = MyPushButton(label_name, class_averages, class_id, self)
                        button.sig_click.connect(self.handle_change)
                        self.widgets.append(button)
                    button.setIconSize(QSize(50, 50))
                    button.setToolTip('Class averages: {}\nClass id: {}'.format(class_averages, class_id))
                    button.setIcon(QIcon(file_name))
                    button.setStyleSheet('QPushButton {color: rgba(0, 0, 0 ,0); background-color: rgba(0, 0, 0, 0); border: 0px; border-color: rgba(0, 0, 0, 0); min-width: 50px; max-width: 50px; min-height: 50px; max-height: 50px}')
                    button_dict.setdefault(
                        class_averages,
                        []
                        ).append(button)
                    self.current_index += 1

        try:
            with open(self.settings_file, 'r') as read:
                self.button_dict = json.load(read)
        except FileNotFoundError:
            pass

        for file_name, buttons in button_dict.items():
            try:
                update = os.path.getmtime(file_name) < os.path.getmtime(self.settings_file)
            except FileNotFoundError:
                update = False

            for button in buttons:
                if update:
                    try:
                        button.current_layout = self.button_dict[self.combo_text.currentText()][file_name][str(button.isac_class_id)]
                    except KeyError:
                        pass
                self.add_to_layout(
                    button.current_layout,
                    just_add=button,
                    )

    @pyqtSlot(object, object)
    def handle_change(self, sender, event):
        if sender.current_layout == self.neutral_name:
            if event.button() == Qt.LeftButton:
                self.switch_layout(sender, self.good_name)

            elif event.button() == Qt.RightButton:
                self.switch_layout(sender, self.bad_name)
        else:
            self.switch_layout(sender, self.neutral_name)

    @pyqtSlot()
    def adjust_all_layouts(self):
        try:
            self.columns = int(self.edit.text())
        except Exception as e:
            print(e)
            self.edit.setText(str(self.columns))
            return
        for name in self.layouts:
            self.add_to_layout(name)

    def add_to_layout(self, layout_name, add=None, remove=None, clear=False, just_add=None):
        if just_add:
            self.widgets_dict[layout_name].append(just_add)
            self.layouts[layout_name].addWidget(just_add)
            return

        layout_count = self.layouts[layout_name].count()
        for i in reversed(range(layout_count)):
            item = self.layouts[layout_name].itemAt(i)
            if isinstance(item, QSpacerItem):
                self.layouts[layout_name].removeItem(item)
            elif isinstance(item, QWidgetItem):
                self.layouts[layout_name].removeItem(item)
                #if clear:
                #    item.widget().setParent(None)
            elif isinstance(item, QHBoxLayout):
                for j in reversed(range(item.count())):
                    item_j = item.itemAt(j)
                    if isinstance(item_j, QSpacerItem):
                        item.removeItem(item_j)
                        continue
                    elif isinstance(item_j, QWidgetItem):
                        pass
                        #if clear:
                        #    item_j.widget().setParent(None)
                    elif item_j is None:
                        pass
                    else:
                        assert False, item_j
                self.layouts[layout_name].removeItem(item)
            else:
                assert False, item

        if clear:
            self.widgets_dict[layout_name] = []
            self.current_index = 0
            return
        if remove is not None:
            self.widgets_dict[layout_name].remove(remove)
        if add is not None:
            self.widgets_dict[layout_name].append(add)

        self.labels[layout_name].setText(
            '{}: {}'.format(layout_name, len(self.widgets_dict[layout_name]))
            )
        a = np.array(
            [
                (entry.isac_class_averages, entry.isac_class_id)
                for entry in self.widgets_dict[layout_name]
                ],
            dtype='U5000,i8'
            )
        indices = np.argsort(a, order=['f0', 'f1'])
        self.widgets_dict[layout_name] = np.array(
            self.widgets_dict[layout_name], dtype=np.object
            )[indices].tolist()

        layout = None
        for i in range(len(self.widgets_dict[layout_name])):
            i_hor = i % self.columns
            if i_hor == 0:
                if layout is not None:
                    layout.addStretch(1)
                layout = QHBoxLayout()
                self.layouts[layout_name].addLayout(layout)

            layout.addWidget(self.widgets_dict[layout_name][i])
            self.button_dict.setdefault(
                self.combo_text.currentText(), {}
                ).setdefault(
                    self.widgets_dict[layout_name][i].isac_class_averages,
                    {}
                    )[str(self.widgets_dict[layout_name][i].isac_class_id)] = layout_name

        if layout is not None:
            layout.addStretch(1)
        self.layouts[layout_name].addStretch(1)

        if self.widgets_dict[layout_name]:
            with open(self.settings_file, 'w') as write:
                json.dump(self.button_dict, write, indent=1)
        return self.widgets_dict[layout_name]

    def clear(self):
        for name in self.layouts:
            self.add_to_layout(name, clear=True)

    def switch_layout(self, sender, layout_name):
        self.add_to_layout(sender.current_layout, remove=sender)
        self.add_to_layout(layout_name, add=sender)
        sender.current_layout = layout_name

    @pyqtSlot()
    def retrain(self):
        time_string = datetime.datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
        classes_folder = self.classes_folder.format(time_string)
        good_folder = self.good_folder.format(time_string)
        bad_folder = self.bad_folder.format(time_string)
        model_out = self.model_out.format(time_string)
        config_out = self.config_out.format(time_string)

        original_folder = os.path.join(classes_folder, self.input_name)
        tu.mkdir_p(original_folder)

        self.current_model = model_out

        config_file = os.path.join(
            os.path.dirname(__file__),
            'templates',
            'cinderella_config.json'
            )
        with open(config_file, 'r') as read:
            content = read.read()
        content = content.replace('XXXGOODXXX', good_folder)
        content = content.replace('XXXBADXXX', bad_folder)
        content = content.replace('XXXMODELXXX', model_out)
        with open(config_out, 'w') as write:
            write.write(content)

        for current_name, out_dir_classes in (
                (self.bad_name, bad_folder),
                (self.good_name, good_folder)
                ):
            os.makedirs(out_dir_classes, exist_ok=True)
            index_dict = {}
            widgets = self.add_to_layout(current_name)
            for widget in widgets:
                index_dict.setdefault(widget.isac_class_averages, []).append(
                    widget.isac_class_id
                    )
            for file_name, index_list in index_dict.items():
                out_filename = file_name.replace(self.log_folder, '').replace(self.project_folder, '').replace('/', '_')
                out_symlink = os.path.join(original_folder, out_filename)
                print(out_filename)
                print(out_symlink)
                if not os.path.islink(out_symlink):
                    tu.symlink_rel(file_name, out_symlink)

                out_file = os.path.join(
                    classes_folder,
                    '{}_{}_list.txt'.format(out_filename, current_name)
                    )
                with open(out_file, 'w') as write:
                    write.write('\n'.join(map(str, index_list)))
                cmd = '{} {} {} --list={}'.format(
                    self.e2proc2d_exec,
                    file_name,
                    os.path.join(
                        out_dir_classes,
                        out_filename
                        ),
                    out_file
                    )
                print('Execute:', cmd)
                try:
                    subprocess.call(cmd.split())
                except Exception as e:
                    print(e)
        cmd = '{} -c {}'.format(self.sp_cinderella_train_exec, config_out)
        print('Execute:', cmd)
        try:
            idx = subprocess.call(cmd.split())
        except Exception as e:
            print(e)
        else:
            if not idx:
                self.repick(time_string)

    @pyqtSlot()
    def repick(self, time_string=None):
        try:
            if time_string is None:
                classes_folder = os.path.join(self.log_folder, self.combo_text.currentText().rsplit('_', 1)[0])
            else:
                classes_folder = self.classes_folder.format(time_string)
        except ValueError:
            tu.message('Could not find model for current entry: {}.\nPlease provide a repicking run as starting point.'.format(self.combo_text.currentText()))
            return

        if not os.path.isdir(classes_folder):
            print(classes_folder)
            tu.message('Could not find model for current combo: {}.\nPlease provide a repicking run as starting point.'.format(self.combo_text.currentText()))
            return

        original_folder = os.path.join(classes_folder, self.input_name)
        model_out = os.path.join(classes_folder, self.model_name)
        threshold = float(self.threshold_repick.text())
        output_folder = '{}_{}'.format(classes_folder, threshold)

        if not os.path.isfile(model_out):
            print(classes_folder)
            tu.message('Could not find model for current combo: {}.\nPlease provide a repicking run as starting point.'.format(self.combo_text.currentText()))
            return

        try:
            shutil.rmtree(output_folder)
        except Exception:
            pass

        cmd = '{} -i {} -o {} -w {} -t {}'.format(
            self.sp_cinderella_predict_exec,
            original_folder,
            output_folder,
            model_out,
            threshold,
            )
        print('Execute:', cmd)
        try:
            subprocess.call(cmd.split())
        except Exception as e:
            print(e)

        for entry in glob.glob(os.path.join(output_folder, '*.txt')):
            if not entry.endswith('_index_confidence.txt'):
                continue

            file_name = entry.rsplit('_index_confidence.txt', 1)[0]
            for suffix in ('_good', '_bad'):
                out_folder = os.path.join(output_folder, 'png{}'.format(suffix))
                in_file = '{}{}.hdf'.format(file_name, suffix)
                tu.mkdir_p(out_folder)
                cmd = '{} {} {} --outmode=uint16 --unstacking'.format(
                    self.e2proc2d_exec,
                    in_file,
                    os.path.join(
                        out_folder,
                        '{}.png'.format(os.path.basename(in_file))
                        ),
                    )
                print('Execute:', cmd)
                try:
                    subprocess.call(cmd.split())
                except Exception as e:
                    print(e)

        self.add_combo_item([os.path.basename(output_folder)])
        cur_text = self.combo_text.blockSignals(True)
        self.combo_text.setCurrentText(os.path.basename(output_folder))
        self.combo_text.blockSignals(cur_text)
        self.combo_text.currentTextChanged.emit(os.path.basename(output_folder))

    @pyqtSlot()
    def confirm(self):
        self.sig_new_config.emit(self.model_out, self.threshold)


class MyPushButton(QPushButton):
    sig_click = pyqtSignal(object, object)

    def __init__(self, layout, isac_file_name, isac_id, *args, **kwargs):
        super(MyPushButton, self).__init__(*args, **kwargs)
        self.isac_class_id = isac_id
        self.isac_class_averages = isac_file_name
        self.current_layout = layout

    def mousePressEvent(self, event):
        if event.button() in (Qt.RightButton, Qt.LeftButton):
            self.sig_click.emit(self, event)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import shutil
import os
import pexpect as pe
from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal


class MountCalculator(QObject):
    """
    MountCalculator object.

    Inherits from:
    QObject

    Buttons:
    None

    Signals:
    sig_finished - Signal emitted, if process finished (str, str, str)
    """
    sig_finished = pyqtSignal(str, str, str)

    def __init__(self, name, parent=None):
        """
        Initialize object variables.

        Arguments:
        name - Name of the mount point
        parent - Parent widget (default None)

        Return:
        None
        """
        super(MountCalculator, self).__init__(parent)
        self.name = name
        self.ssh_dict = None
        self.quota_command_dict = None
        self.password_dict = None
        self.kill_thread = False
        self.running = False

    @pyqtSlot(str, str)
    def calculate_df_quota(self, key, mount_folder):
        """
        Calculate the quota with the help of the df command.

        Arguments:
        key - Mount point key
        mount_folder - Mount folder

        Return:
        None
        """
        if key.startswith('HDD') and len(key) == 5:
            old_key = key
            key = key[:-2]
        else:
            old_key = key
            key = key

        if key != self.name.replace(' ', '_'):
            return None
        else:
            pass

        if old_key.startswith('HDD') and len(old_key) == 5:
            key = old_key
        else:
            pass

        self.running = True
        try:
            total_quota = shutil.disk_usage(mount_folder).total / 1e12
            used_quota = shutil.disk_usage(mount_folder).used / 1e12
        except FileNotFoundError:
            self.sig_finished.emit(
                'Needs remount',
                key,
                'red'
                )
        else:
            if total_quota == used_quota and total_quota == 0:
                self.sig_finished.emit(
                    'Needs remount',
                    key,
                    'red'
                    )
            else:
                self.sig_finished.emit(
                    '{0:.1f}TB / {1:.1f}TB'.format(used_quota, total_quota),
                    key,
                    'lightgreen'
                    )
        self.running = False

    @pyqtSlot(str, str, str, str, object, object, object)
    def calculate_ssh_quota(
            self, user, folder, device, mount_folder, ssh_dict,
            quota_command_dict, password_dict
            ):
        """
        Calculate the quota via ssh.

        Arguments:
        mount_folder - Mount folder
        user - Username
        folder - Folder to mount
        device - Device name
        ssh_dict - ssh_dict containing ssh information
        quota_command_dict - Dictionary containing the quota commands
        password_dict - Dictionary containing passwords

        Return:
        None
        """
        if device != self.name.replace(' ', '_'):
            return None
        else:
            pass

        self.running = True
        self.ssh_dict = ssh_dict
        self.quota_command_dict = quota_command_dict
        self.password_dict = password_dict

        try:
            used_quota, total_quota = self.get_ssh_quota(
                user=user,
                folder=folder,
                device=device
                )
        except KeyError:
            total_quota = shutil.disk_usage(mount_folder).total / 1e12
            used_quota = shutil.disk_usage(mount_folder).used / 1e12
        except pe.exceptions.TIMEOUT:
            total_quota = shutil.disk_usage(mount_folder).total / 1e12
            used_quota = shutil.disk_usage(mount_folder).used / 1e12

        self.sig_finished.emit(
            '{0:.1f}TB / {1:.1f}TB'.format(used_quota, total_quota),
            device,
            'lightgreen'
            )
        self.running = False

    @pyqtSlot(str, str, str)
    def calculate_get_quota(self, key, quota, mount_folder):
        """
        Calculate the quota by calculating the size of every file.

        Arguments:
        key - Mount point key
        mount_folder - Mount folder
        quota - User provided maximum quota

        Return:
        None
        """
        if key != self.name.replace(' ', '_'):
            return None
        else:
            pass

        self.running = True
        total_quota = float(quota)
        try:
            used_quota = self.get_folder_size(mount_folder, 0) / 1024 ** 4
        except PermissionError:
            self.sig_finished.emit(
                'DENIED',
                key,
                'red'
                )
        except FileNotFoundError:
            print(mount_folder, 'Directory changed during quota estimation! Wait for next run!')
            self.sig_finished.emit(
                'CHANGED',
                key,
                'red'
                )
        except OSError:
            print(mount_folder, 'Please remount!')
            self.sig_finished.emit(
                'Needs remount',
                key,
                'red'
                )
        else:
            self.sig_finished.emit(
                '{0:.1f}TB / {1:.1f}TB'.format(used_quota, total_quota),
                key,
                'lightgreen'
                )
        self.running = False

    def get_ssh_quota(self, user, folder, device):
        """
        Get the quota via ssh command.

        Arguments:
        user - User name
        folder - Mounted folder
        device - Device name

        Return:
        None
        """
        command = 'ssh {0}@{1} {2}'.format(
            user,
            self.ssh_dict[device],
            self.quota_command_dict[device]
            )
        child = pe.spawnu(command)

        try:
            idx = child.expect(
                [
                    "{0}@{1}'s password:".format(user, self.ssh_dict[device]),
                    'RSA.*'
                    ],
                timeout=4
                )
        except pe.exceptions.TIMEOUT:
            print('SSH quota command failed! Mount point might be unavailable.')
            raise
        except pe.exceptions.EOF:
            print('SSH quota command failed! Mount point might be unavailable.')
            raise

        if idx == 0:
            child.sendline(self.password_dict[device])
        elif idx == 1:
            child.sendline('yes')
            child.expect([
                "{0}@{1}'s password:".format(user, self.ssh_dict[device]),
                'RSA.*'
                ])
            child.sendline(self.password_dict[device])
        else:
            print('SSH quota command failed!')
            raise pe.exceptions.TIMEOUT

        child.expect(pe.EOF)
        if self.quota_command_dict[device].startswith('quota'):
            used_quota, total_quota = self.get_quota_quota_command(
                text=child.before.split('\n'),
                folder=folder
                )
        else:
            print('To get the quota via SSH failed, do not know how to handle {0}'.format(
                self.quota_command_dict[device]
                ))
            print('Command:\n{0}'.format(child.before))
            print('Please write a wrapper for this case or write the content to the author of TranSPHIRE')
            raise pe.exceptions.TIMEOUT
        return used_quota, total_quota

    @staticmethod
    def get_quota_quota_command(text, folder):
        """
        Extract the quota from the quota command.

        Arguments:
        text - Text returned by quota command.
        folder - Mounted folder

        Return:
        None
        """
        write_value = False
        value_line = None
        for line in text:
            if write_value:
                value_line = line
                write_value = False
            else:
                pass
            if '{0}/'.format(folder.split('/')[0]) in line:
                write_value = True
            else:
                pass

        if value_line is None:
            raise KeyError

        size_list = []
        for value in value_line.split()[:2]:
            unit = value[-1]
            try:
                int(unit)
            except ValueError:
                size = value[:-1]
                if unit.startswith('M'):
                    adjust = 1024**2
                elif unit.startswith('G'):
                    adjust = 1024
                elif unit.startswith('T'):
                    adjust = 1
                elif unit.startswith('P'):
                    adjust = 1/1024
                else:
                    print(
                        unit,
                        'unit of quota command not known!'
                        )
                    print(
                        'Please contact the author of TranSPHIRE to fix this issue'
                        )
                    raise KeyError
            else:
                adjust = 1024**3
                size = value
            size_list.append(float(size) / adjust)
        return size_list[0], size_list[1]

    def get_folder_size(self, folder, size):
        """
        Get the size of the folder recursively

        Arguments:
        folder - Folder to check contents
        size - Current caclulated size

        Return:
        Calculated size
        """
        for entry in os.scandir(folder):
            if self.kill_thread:
                return size
            elif entry.is_dir():
                size = self.get_folder_size(entry.path, size)
            else:
                size += entry.stat().st_size

        return size



import setuptools
import os
import re

here = os.path.abspath(os.path.dirname(__file__))

def get_version():
    init_file = os.path.join(here, 'transphire', '__init__.py')
    with open(init_file) as read:
        data = read.read()
    return re.search("^__version__ = '(.*)'$", data, re.M).group(1)


with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open(os.path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    requirements = f.read().split()
requirements.remove('PyQt5')


setuptools.setup(
    name='transphire',
    version=get_version(),
    description='Automated post data aquisition processing tool',
    long_description=long_description,
    url='https://github.com/mstabrin/transphire',
    author='Markus Stabrin',
    author_email='markus.stabrin@tu-dortmund.de',
    license='GPLv3',
    packages=setuptools.find_packages(exclude=[]),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'transphire = transphire.__main__:run_package'
            ]
        },
    install_requires = requirements,
    classifiers = (
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: POSIX :: Linux',
        'Environment :: X11 Applications :: Qt',
        'Development Status :: 4 - Beta'
        ),
    )




"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import glob
import os
import re
import datetime
import numpy as np
import traceback as tb
import subprocess
from hyperspy.io_plugins.digital_micrograph import DigitalMicrographReader

from . import transphire_import as ti
from . import transphire_utils as tu


def extract_time_and_grid_information(root_name, settings, queue_com, name):
    """
    Extract the time and grid information from the micrograph names.

    Arguments:
    root_name - Name to extract information from.
    settings - TranSPHIRE settings.
    queue_com - Queue for communication.
    name - Name of the process.

    Returns:
    hole_number, spot1_number, spot2_number, date, time
    """
    message = None
    if settings['Input']['Software'] == 'Just Stack':
        hole = 0
        grid_number = 0
        spot1 = 0
        spot2 = 0
        date = 0
        time = 0

    elif settings['Input']['Software'] == 'Latitude S':
        if settings['Input']['Camera'] in ('Falcon2', 'Falcon3'):
            message = '\n'.join([
                'Falcon2/Falcon3 is not supported for Software {0}.'.format(
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

        elif settings['Input']['Camera'] in ('K2', 'K3'):
            search_results = re.search(
                r'(?P<date>\d{8})_(?P<project>.*)_A(?P<atlas>\d{3,})_G(?P<grid>\d{3,})_H(?P<hole>\d{3,})_D(?P<spot>\d{3,})',
                root_name
                )
            grid_number = '{0}_{1}'.format(
                search_results.group('atlas'),
                search_results.group('grid')
                )
            hole = search_results.group('hole')
            spot1 = search_results.group('spot')
            spot2 = ''
            date = search_results.group('date')
            with open('{0}.gtg'.format(root_name), 'rb') as read:
                reader = DigitalMicrographReader(read)
                reader.parse_file()
            time = datetime.datetime.strptime(
                reader.tags_dict['DataBar']['Acquisition Time'],
                '%I:%M:%S %p'
                ).strftime('%H%M%S')

        else:
            message = '\n'.join([
                'Camera {0} for Software {1} not known!'.format(
                    settings['Input']['Camera'],
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.8':

        if settings['Input']['Camera'] == 'Falcon2' \
                or settings['Input']['Camera'] == 'Falcon3' \
                or settings['Input']['Camera'] in ('K2', 'K3'):
            *skip, grid, skip, old_file = \
                os.path.realpath(root_name).split('/')
            grid_number = grid.split('_')[1]
            *skip, hole, skip, spot1, spot2, date, time = old_file.split('_')
            del skip

        else:
            message = '\n'.join([
                'Camera {0} for Software {1} not known!'.format(
                    settings['Input']['Camera'],
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.9':

        if settings['Input']['Camera'] == 'Falcon2' \
                or settings['Input']['Camera'] == 'Falcon3' \
                or settings['Input']['Camera'] in ('K2', 'K3'):
            *skip, grid, skip, old_file = \
                os.path.realpath(root_name).split('/')
            grid_number = grid.split('_')[1]
            *skip, hole, skip, spot1, spot2, date, time = old_file.split('_')
            del skip

        elif settings['Input']['Camera'] in ('K2', 'K3'):
            *skip, grid, skip, old_file = \
                os.path.realpath(root_name).split('/')
            grid_number = grid.split('_')[1]
            *skip, hole, skip, spot1, spot2, date, time = old_file.split('_')
            del skip

        else:
            message = '\n'.join([
                'Camera {0} for Software {1} not known!'.format(
                    settings['Input']['Camera'],
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

    else:
        message = '\n'.join([
            'Software {0} not known!'.format(settings['Input']['Software']),
            'Please contact the TranSPHIRE authors!'
            ])

    if message is None:
        pass
    else:
        queue_com['error'].put(message, name)
        raise IOError(message)

    return hole, grid_number, spot1, spot2, date, time


def find_frames(frames_root, compare_name, settings, queue_com, name, write_error):
    """
    Find frames based on Software, Type and camera used.

    frames_root - Root name of the frames.
    compare_name - Name to compare jpg and frames as time might differ.
    settings - TranSPHIRE settings used.
    queue_com - Queue for communication
    name - Name of process
    write_error - Write error function

    Returns:
    None if the number of frames does not match user input.
    False if an error occured and the file needs to be skipped.
    True if the function was successful.
    """
    message = None
    if settings['Input']['Software'] == 'Just Stack':
        frames = glob.glob(
            '{0}.{1}'.format(
                compare_name,
                settings['Input']['Input frames extension']
                )
            )
        if not frames:
            return False

        try:
            value, _, _, checked_nr_frames = check_nr_frames(
                frames=frames,
                settings=settings
                )
        except BlockingIOError:
            write_error(
                msg=tb.format_exc(),
                root_name=frames_root
                )
            return False

        if not value:
            message = 'File {0} has {1} frames instead of {2}\n'.format(
                frames[0],
                checked_nr_frames,
                int(settings['Input']['Number of frames'])
                )
            write_error(
                msg=message,
                root_name=frames_root
                )
            return None
        else:
            return True
    elif settings['Input']['Software'] == 'Latitude S':
        if settings['Input']['Type'] == 'Frames':
            message = '\n'.join([
                'Frames and Falcon2/Falcon3 is not supported for Software {0}.'.format(
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

        elif settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('Falcon2', 'Falcon3'):
                message = '\n'.join([
                    'Falcon2/Falcon3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] in ('K2', 'K3'):
                frames = glob.glob(
                    '{0}.{1}'.format(
                        os.path.join(os.path.dirname(compare_name), 'Stack', os.path.basename(compare_name)),
                        settings['Input']['Input frames extension']
                        )
                    )

                if len(frames) != 1:
                    message = 'File {0} has {1} movie files instead of 1\n'.format(
                        frames_root,
                        len(frames)
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    try:
                        value, _, _, checked_nr_frames = check_nr_frames(
                            frames=frames,
                            settings=settings
                            )
                    except BlockingIOError:
                        write_error(
                            msg=tb.format_exc(),
                            root_name=frames_root
                            )
                        return False

                if not value:
                    message = 'File {0} has {1} frames instead of {2}\n'.format(
                        frames[0],
                        checked_nr_frames,
                        int(settings['Input']['Number of frames'])
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    return None
                else:
                    return True

            else:
                message = '\n'.join([
                    'Camera {0} for Software {1} not known!'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.8':

        ####
        #
        # FRAMES
        #
        ####

        if settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                frames = glob.glob(
                    '{0}-*.{1}'.format(
                        frames_root,
                        settings['Input']['Input frames extension']
                        )
                    )
                if len(frames) != int(settings['Input']['Number of frames']):
                    write_error(
                        msg='File {0} has {1} movie files instead of {2}\n'.format(
                            frames_root,
                            len(frames),
                            settings['Input']['Number of frames']
                            ),
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    return True

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Stack
        #
        ####

        elif settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Stack and K2, K3 is not supported in EPU >=1.8 version',
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                frames = glob.glob(
                    '{0}*_Fractions.{1}'.format(
                        compare_name,
                        settings['Input']['Input frames extension']
                        )
                    )

                if len(frames) != 1:
                    message = 'File {0} has {1} movie files instead of 1\n'.format(
                        frames_root,
                        len(frames)
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    try:
                        value, _, _, checked_nr_frames = check_nr_frames(
                            frames=frames,
                            settings=settings
                            )
                    except BlockingIOError:
                        write_error(
                            msg=tb.format_exc(),
                            root_name=frames_root
                            )
                        return False

                if not value:
                    message = 'File {0} has {1} frames instead of {2}\n'.format(
                        frames[0],
                        checked_nr_frames,
                        int(settings['Input']['Number of frames'])
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    return None
                else:
                    return True

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.9':

        ####
        #
        # FRAMES
        #
        ####

        if settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Frames and K2, K3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Stack
        #
        ####

        elif settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2'):
                raw_frames = glob.glob(
                    '{0}*-*.{1}'.format(
                        compare_name,
                        settings['Input']['Input frames extension']
                        )
                    )
                frames_re = re.compile('{0}.*-[0-9]+.{1}'.format(
                        compare_name,
                        settings['Input']['Input frames extension']
                        ))
                frames = [frame for frame in raw_frames if frames_re.match(frame) is not None]

                if len(frames) != 1:
                    message = 'File {0} has {1} movie files instead of 1\n'.format(
                        frames_root,
                        len(frames)
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    try:
                        value, _, _, checked_nr_frames = check_nr_frames(
                            frames=frames,
                            settings=settings
                            )
                    except BlockingIOError:
                        write_error(
                            msg=tb.format_exc(),
                            root_name=frames_root
                            )
                        return False

                if not value:
                    message = 'File {0} has {1} frames instead of {2}\n'.format(
                        frames[0],
                        checked_nr_frames,
                        int(settings['Input']['Number of frames'])
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    return None
                else:
                    return True

            elif settings['Input']['Camera'] == 'K3':
                frames = glob.glob(
                    '{0}*_fractions.{1}'.format(
                        compare_name,
                        settings['Input']['Input frames extension']
                        )
                    )

                if len(frames) != 1:
                    message = 'File {0} has {1} movie files instead of 1\n'.format(
                        frames_root,
                        len(frames)
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    try:
                        value, _, _, checked_nr_frames = check_nr_frames(
                            frames=frames,
                            settings=settings
                            )
                    except BlockingIOError:
                        write_error(
                            msg=tb.format_exc(),
                            root_name=frames_root
                            )
                        return False

                if not value:
                    message = 'File {0} has {1} frames instead of {2}\n'.format(
                        frames[0],
                        checked_nr_frames,
                        int(settings['Input']['Number of frames'])
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    return None
                else:
                    return True

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                frames = glob.glob(
                    '{0}*_Fractions.{1}'.format(
                        compare_name,
                        settings['Input']['Input frames extension']
                        )
                    )

                if len(frames) != 1:
                    message = 'File {0} has {1} movie files instead of 1\n'.format(
                        frames_root,
                        len(frames)
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    if len(frames) == 0:
                        return False
                    else:
                        return None
                else:
                    try:
                        value, _, _, checked_nr_frames = check_nr_frames(
                            frames=frames,
                            settings=settings
                            )
                    except BlockingIOError:
                        write_error(
                            msg=tb.format_exc(),
                            root_name=frames_root
                            )
                        return False

                if not value:
                    message = 'File {0} has {1} frames instead of {2}\n'.format(
                        frames[0],
                        checked_nr_frames,
                        int(settings['Input']['Number of frames'])
                        )
                    write_error(
                        msg=message,
                        root_name=frames_root
                        )
                    return None
                else:
                    return True

            else:
                message = '\n'.join([
                    'Camera {0} not known for Softwaee {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    else:
        message = '\n'.join([
            '{0}: Unknown Software!'.format(settings['Input']['Software']),
            'Please contact the TranSPHIRE authors!'
            ])

    assert bool(message is not None)
    queue_com['error'].put(
        message,
        name
        )
    raise IOError(message)


def get_x_dim(frames, settings):
    command = "{0} '{1}'".format(
        settings['Path']['IMOD header'],
        frames[0]
        )

    text = subprocess.check_output(command, shell=True, encoding='utf-8')

    x_dim = 0
    for line in text.split('\n'):
        if line.startswith(' Number of columns, rows, sections .....'):
            x_dim = int(line.split()[-3])

    return int(x_dim)


def check_nr_frames(frames, settings, force=False):
    """
    Check if the nr of frames of the stack match the given nr of frames

    Arguments:
    frames - List of found frames
    settings - TranSPHIRE settings
    """
    if int(settings['Input']['Number of frames']) == -1 and not force:
        return True, 0, 0, 0

#    if settings['is_superres'].value == 2:
#        settings['is_superres'].value = bool(get_x_dim(frames, settings) > 7000)
#
    if not frames:
        frames.append(None)
        return False, 0, 0, 0
    else:
        command = "{0} '{1}'".format(
            settings['Path']['IMOD header'],
            frames[0]
            )

        text = subprocess.check_output(command, shell=True, encoding='utf-8')

        z_dim = 0
        x_dim = 0
        y_dim = 0
        for line in text.split('\n'):
            if line.startswith(' Number of columns, rows, sections .....'):
                x_dim, y_dim, z_dim = list(map(int, line.split()[-3:]))

        return bool(z_dim == int(settings['Input']['Number of frames'])), x_dim, y_dim, z_dim


def find_related_frames_to_jpg(frames_root, root_name, settings, queue_com, name):
    """
    Find related frames to the jpg file based on the used software.

    Arguments:
    frames_root - Root name to search for related files
    root_name - Root name of the jpg file
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of the process
    """
    message = None
    if settings['Input']['Software'] == 'Just Stack':
        compare_name_frames = root_name
        compare_name_meta = root_name
        frames = glob.glob('{0}*.{1}'.format(
            compare_name_frames,
            settings['Input']['Input frames extension']
            ))
        return frames, compare_name_frames, compare_name_meta

    elif settings['Input']['Software'] == 'Latitude S':
        if settings['Input']['Type'] == 'Frames':
            message = '\n'.join([
                'Frames and Falcon2/Falcon3 is not supported for Software {0}.'.format(
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

        elif settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('Falcon2', 'Falcon3'):
                message = '\n'.join([
                    'Falcon2/Falcon3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] in ('K2', 'K3'):
                compare_name_frames = os.path.join(os.path.dirname(root_name), 'Stack', os.path.basename(root_name))
                compare_name_meta = root_name
                frames = glob.glob('{0}.{1}'.format(
                    compare_name_frames,
                    settings['Input']['Input frames extension']
                    ))
                return frames, compare_name_frames, compare_name_meta

            else:
                message = '\n'.join([
                    'Camera {0} for Software {1} not known!'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.8':

        ####
        #
        # Stack
        #
        ####

        if settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Stack and K2, K3 is not supported in EPU >=1.8 version',
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                compare_name_frames = frames_root[:-len('_19911213_2019')]
                compare_name_meta = root_name[:-len('_19911213_2019')]
                frames = glob.glob('{0}*_Fractions.{1}'.format(
                    compare_name_frames,
                    settings['Input']['Input frames extension']
                    ))
                return frames, compare_name_frames, compare_name_meta

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Frames
        #
        ####

        elif settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                frames = sorted(glob.glob(
                    '{0}-*.{1}'.format(
                        frames_root,
                        settings['Input']['Input frames extension']
                        )
                    ))
                return frames, frames_root, root_name

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported',
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.9':

        ####
        #
        # Stack
        #
        ####

        if settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2'):
                compare_name_frames = frames_root[:-len('_19911213_2019')]
                compare_name_meta = root_name[:-len('_19911213_2019')]
                raw_frames = glob.glob('{0}*-*.{1}'.format(
                    compare_name_frames,
                    settings['Input']['Input frames extension']
                    ))
                frames_re = re.compile('{0}.*-[0-9]+.{1}'.format(
                        compare_name_frames,
                        settings['Input']['Input frames extension']
                        ))
                frames = [frame for frame in raw_frames if frames_re.match(frame) is not None]
                return frames, compare_name_frames, compare_name_meta

            elif settings['Input']['Camera'] == 'K3':
                compare_name_frames = frames_root[:-len('_19911213_2019')]
                compare_name_meta = root_name[:-len('_19911213_2019')]
                frames = glob.glob('{0}*_fractions.{1}'.format(
                    compare_name_frames,
                    settings['Input']['Input frames extension']
                    ))
                return frames, compare_name_frames, compare_name_meta

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                compare_name_frames = frames_root[:-len('_19911213_2019')]
                compare_name_meta = root_name[:-len('_19911213_2019')]
                frames = glob.glob('{0}*_Fractions.{1}'.format(
                    compare_name_frames,
                    settings['Input']['Input frames extension']
                    ))
                return frames, compare_name_frames, compare_name_meta

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Frames
        #
        ####

        elif settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Frames and K2, K3 is not supported',
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported',
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    else:
        message = '\n'.join([
            '{0}: Unknown Software!'.format(settings['Input']['Software']),
            'Please contact the TranSPHIRE authors!'
            ])

    assert bool(message is not None)
    queue_com['error'].put(
        message,
        name
        )
    raise IOError(message)


def get_copy_command_for_frames(settings, queue_com, name):
    """
    Copy the frames based on stack or frames type.

    settings - TranSPHIRE settings.
    queue_com - Queue for communication.
    name - Name of the process.

    Returns:
    Command to use for copy.
    """
    message = None
    if settings['Copy']['Delete data after import?'] == 'Symlink':
        return "ln -rsf"
    elif settings['Input']['Software'] == 'Just Stack':
        return "rsync --copy-links"
    elif settings['Input']['Software'] == 'Latitude S':
        if settings['Input']['Type'] == 'Frames':
            message = '\n'.join([
                'Frames and Falcon2/Falcon3 is not supported for Software {0}.'.format(
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

        elif settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('Falcon2', 'Falcon3'):
                message = '\n'.join([
                    'Falcon2/Falcon3 is not supported for Software {0}.'.format(
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] in ('K2', 'K3'):
                return 'rsync --copy-links'

            else:
                message = '\n'.join([
                    'Camera {0} for Software {1} not known!'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.8':

        ####
        #
        # Stack
        #
        ####

        if settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Stack and K2, K3 is not supported in EPU >=1.8 version',
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                return 'rsync --copy-links'

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Frames
        #
        ####

        elif settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                return settings['Path']['IMOD newstack']

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported, yet.',
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.9':

        ####
        #
        # Stack
        #
        ####

        if settings['Input']['Type'] == 'Stack':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                return 'rsync --copy-links'

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                return 'rsync --copy-links'

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        ####
        #
        # Frames
        #
        ####

        elif settings['Input']['Type'] == 'Frames':

            if settings['Input']['Camera'] in ('K2', 'K3'):
                message = '\n'.join([
                    'Frames and K2, K3 is not supported in EPU >=1.9',
                    'Please contact the TranSPHIRE authors!'
                    ])

            elif settings['Input']['Camera'] == 'Falcon2' or \
                    settings['Input']['Camera'] == 'Falcon3':
                message = '\n'.join([
                    'Frames and Falcon2/Falcon3 is not supported n EPU >=1.9',
                    'Please contact the TranSPHIRE authors!'
                    ])

            else:
                message = '\n'.join([
                    'Camera {0} not known for Software {1} with type {2}.'.format(
                        settings['Input']['Camera'],
                        settings['Input']['Software'],
                        settings['Input']['Type']
                        ),
                    'Please contact the TranSPHIRE authors!'
                    ])

        else:
            message = '\n'.join([
                '{0}: Unknown Type!'.format(settings['Input']['Type']),
                'Please contact the TranSPHIRE authors!'
                ])

    else:
        message = '\n'.join([
            '{0}: Unknown Software!'.format(settings['Input']['Software']),
            'Please contact the TranSPHIRE authors!'
            ])

    assert bool(message is not None)
    queue_com['error'].put(
        message,
        name
        )
    raise IOError(message)


def find_all_files(root_name, compare_name_frames, compare_name_meta, settings, queue_com, name):
    """
    Find other files that relate to root_name.

    root_name - Root name of files to find.
    compare_name_meta - Name of the meta data to find.
    queue_com - Queue for communication.
    name - Name of the process

    Returns:
    list of files related to root_name.
    """
    message = None
    if settings['Input']['Software'] == 'Just Stack':
        meta_files = glob.glob('{0}.*'.format(root_name))
        frame_files = glob.glob('{0}*'.format(compare_name_frames))
        return set(meta_files), set(frame_files)
    elif settings['Input']['Software'] == 'Latitude S':

        if settings['Input']['Camera'] in ('Falcon2', 'Falcon3'):
            message = '\n'.join([
                'Falcon2/Falcon3 is not supported for Software {0}.'.format(
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

        elif settings['Input']['Camera'] in ('K2', 'K3'):
            meta_files = glob.glob('{0}.*'.format(root_name))
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        else:
            message = '\n'.join([
                'Camera {0} for Software {1} not known!'.format(
                    settings['Input']['Camera'],
                    settings['Input']['Software']
                    ),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.8':

        if settings['Input']['Camera'] in ('K2', 'K3'):
            meta_files = glob.glob('{0}.*'.format(root_name))
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        elif settings['Input']['Camera'] == 'Falcon2' or \
                settings['Input']['Camera'] == 'Falcon3':
            meta_files = [
                name for name in glob.glob('{0}*'.format(compare_name_meta))
                if 'Fractions' not in name
                ]
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        else:
            message = '\n'.join([
                '{0}: Unknown Camera!'.format(settings['Input']['Camera']),
                'Please contact the TranSPHIRE authors!'
                ])

    elif settings['Input']['Software'] == 'EPU >=1.9':

        if settings['Input']['Camera'] in ('K2'):
            meta_files = glob.glob('{0}*'.format(compare_name_meta))
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        elif settings['Input']['Camera'] == 'K3':
            meta_files = [
                name for name in glob.glob('{0}*'.format(compare_name_meta))
                if 'fractions' not in name
                ]
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        elif settings['Input']['Camera'] == 'Falcon2' or \
                settings['Input']['Camera'] == 'Falcon3':
            meta_files = [
                name for name in glob.glob('{0}*'.format(compare_name_meta))
                if 'Fractions' not in name
                ]
            frame_files = glob.glob('{0}*'.format(compare_name_frames))
            return set(meta_files), set(frame_files)

        else:
            message = '\n'.join([
                '{0}: Unknown Camera!'.format(settings['Input']['Camera']),
                'Please contact the TranSPHIRE authors!'
                ])

    else:
        message = '\n'.join([
            '{0}: Unknown Software!'.format(settings['Input']['Software']),
            'Please contact the TranSPHIRE authors!'
            ])

    assert bool(message is not None)
    queue_com['error'].put(
        message,
        name
        )
    raise IOError(message)


def check_outputs(zero_list, non_zero_list, exists_list, folder, command):
    """
    Check, if the output files are present and have the proper size.

    zero_list - List of files that should be zero
    non_zero_list - List of files that should not be zero

    Returns:
    None
    """
    template = '{0} - Command failed:\n{1}\nfor\n{2}.\nPlease check the logfiles in\n{3}!'
    for file_path in non_zero_list:
        try:
            size = os.path.getsize(file_path)
        except OSError as e:
            raise Exception(
                template.format(
                    str(e),
                    command,
                    file_path,
                    folder
                    )
                )
        else:
            if size == 0:
                raise Exception(
                    template.format(
                        'SIZE ERROR 1',
                        command,
                        file_path,
                        folder
                        )
                    )
            else:
                continue

    for file_path in zero_list:
        try:
            size = os.path.getsize(file_path)
        except OSError as e:
            raise Exception(
                template.format(
                    str(e),
                    command,
                    file_path,
                    folder
                    )
                )
        else:
            if size > 0:
                raise Exception(
                    template.format(
                        'SIZE ERROR 2',
                        command,
                        file_path,
                        folder
                        )
                    )
            else:
                continue

    for file_path in exists_list:
        if not os.path.exists(file_path):
            raise Exception(
                template.format(
                    'Exist error',
                    command,
                    file_path,
                    folder
                    )
                )
        else:
            continue


def check_for_outlier(dict_name, data, file_name, settings):
    dtype_dict = ti.get_dtype_dict()
    lower_median = int(settings['Notification']['Nr. of values used for median'])
    warning_list = []
    skip_list = []

    file_name_match = os.path.basename(os.path.splitext(file_name)[0])
    match_file = re.compile(file_name_match)
    vmatch = np.vectorize(lambda x:bool(match_file.search(x)))
    try:
        mask = vmatch(data['file_name'])
    except ValueError:
        print('ERROR with file!')
        print('data', data)
        print('dict_name', dict_name)
        print('file_name', file_name)
        raise

    for (key, dtype) in dtype_dict[dict_name]:
        if dtype not in ('<f8', '<i8'):
            continue

        try:
            last_values_median = np.median(data[key][-lower_median:])
            warning_low, warning_high = settings['Notification']['{0} {1} warning'.format(dict_name, key)].split()
            skip_low, skip_high = settings['Notification']['{0} {1} skip'.format(dict_name, key)].split()
        except KeyError:
            continue
        except Exception as e:
            print('Exception')
            print('lower_median', lower_median)
            print('dict_name', dict_name)
            print('key', key)
            print('data', data)
            print(e)
            continue

        if float(warning_low) <= last_values_median and last_values_median <= float(warning_high):
            pass
        else:
            warning_list.append([
                key,
                last_values_median,
                warning_low,
                warning_high
                ])

        assert data[mask].shape[0] == 1, [data[mask], file_name, file_name_match, data['file_name']]

        if float(skip_low) <= data[key][mask][0] and data[key][mask][0] <= float(skip_high):
            pass
        else:
            skip_list.append([
                key,
                data[key][mask][0],
                skip_low,
                skip_high
                ])

    return warning_list, skip_list


def get_logfiles(log_prefix):
    """
    Return the names of the log_file and error file.

    Arguments:
    log_prefix - Prefix to use.

    Returns:
    Name of log file, Name of error file
    """
    tu.mkdir_p(os.path.dirname(log_prefix))
    template = '{0}_transphire.{{0}}'.format(log_prefix)

    if os.path.exists(template.format('log')):
        log_prefix_faulty = os.path.join(os.path.dirname(log_prefix), 'FAULTY', os.path.basename(log_prefix))
        tu.mkdir_p(os.path.dirname(log_prefix_faulty))
        template_faulty = '{0}_transphire_{{1}}.{{0}}'.format(log_prefix_faulty)
        idx = 0
        while os.path.exists(template_faulty.format('log', idx)):
            idx += 1
        tu.copy(template.format('log'), template_faulty.format('log', idx))
        tu.copy(template.format('err'), template_faulty.format('err', idx))

    return template.format('log'), template.format('err')



from PyQt5.QtWidgets import QWidget, QVBoxLayout, QDialog

from . import tabdocker

class LogViewerDialog(QDialog):


    def __init__(self, parent=None):
        super(LogViewerDialog, self).__init__(parent)

        central_raw_layout = QVBoxLayout(self)
        central_raw_layout.setContentsMargins(0, 0, 0, 0)
        central_widget_raw = QWidget(self)
        central_widget_raw.setObjectName('central_raw')
        central_raw_layout.addWidget(central_widget_raw)

        central_layout = QVBoxLayout(central_widget_raw)
        central_widget = QWidget(self)
        central_widget.setObjectName('central')
        central_layout.addWidget(central_widget)

        layout = QVBoxLayout(central_widget)

        self.tabs = tabdocker.TabDocker(self)
        self.tabs.setObjectName('tab')

        layout.addWidget(self.tabs)

    def add_tab(self, widget, name):
        self.tabs.add_tab(widget, name)




"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import os
import copy
from PyQt5.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QLabel,
    QPushButton,
    QLineEdit,
    QHBoxLayout,
    QComboBox,
    QFileDialog
    )
from PyQt5.QtCore import pyqtSignal, pyqtSlot

from . import transphire_utils as tu
from . import transphire_content as tc
from . import tabdocker


class LoadContent(QWidget):
    """
    LoadContent widget.
    Widget used for the LoadContentContainer.

    Inherits from:
    QWidget

    Signals:
    delete - Emited, when the delete button is pressed (object)
    """
    delete = pyqtSignal(object)

    def __init__(self, typ, separator, settings_folder, hdd=None, parent=None):
        """
        Setup the layout for the widget

        Arguments:
        typ - Typ/Name of content
        separator - Separator widget list to delete if necessary
        hdd - Content is hdd (default None)
        parent - Parent widget (default None)

        Return:
        None
        """
        super(LoadContent, self).__init__(parent)
        # Variables
        self.separator = separator
        self.typ = typ
        self.layout = QVBoxLayout(self)
        self.layout_old = None
        self.content = []
        self.label_dict = {}
        self.layout.setContentsMargins(0, 0, 0, 0)

        self.idx_name = 0
        self.idx_values = 1
        self.idx_dtype = 2
        self.idx_group = 3
        self.idx_type = 4
        self.idx_priority = 5
        self.idx_tooltip = 6
        self.idx_toggle = 7

        self.idx_name_name = 0
        self.idx_name_global = 1

        self.idx_type_type = 0
        self.idx_type_toggle = 1

        # Fill content based on typ
        content_function = tu.get_function_dict()[typ]['content']
        is_license = tu.get_function_dict()[typ]['license']
        items_old = None
        if typ == 'Mount':
            button = QPushButton('Delete', self)
            button.clicked.connect(self._button_clicked)
            self.layout.addWidget(button)
            items = content_function(hdd=hdd)
            if hdd is None:
                pass
            else:
                self.setEnabled(False)
        elif typ in ('Copy', 'Others'):
            items = content_function(settings_folder=settings_folder)
        elif typ in ('Path'):
            items, items_old = content_function()
            tab_widget = tabdocker.TabDocker(self)
            self.layout.addWidget(tab_widget)
            widget = QWidget(self)
            self.layout = QVBoxLayout(widget)
            self.layout.setContentsMargins(0, 0, 0, 0)
            tab_widget.add_tab(widget, 'Current')
            widget = QWidget(self)
            self.layout_old = QVBoxLayout(widget)
            self.layout_old.setContentsMargins(0, 0, 0, 0)
            tab_widget.add_tab(widget, 'Old')
        else:
            items = content_function()

        items.extend([
        ['WIDGETS MAIN', '10', int, '', 'PLAIN', '', ''],
        ['WIDGETS ADVANCED', '10', int, '', 'PLAIN', '', ''],
        ['WIDGETS RARE', '10', int, '', 'PLAIN', '', ''],
        ])
        if is_license:
            items.append(
                ['IMPORTANT', 'THIS SOFTWARE IS NOT PUBLISHED UNDER AN OPEN-SOURCE LICENSE.\nPLEASE CHECK IF YOU NEED/OWN A LICENSE BEFORE USING THIS APPLICATION.', str, '', 'PLAIN', '', ''],
                )


        # Fill widget with content items
        self._fill_default(items, items_old)

    def _fill_default(self, items, items_old):
        """
        Fill the widget with content.

        Arguments:
        items - Content for the widget as list

        Return:
        None
        """
        # Layout
        for current_layout, items in zip([self.layout, self.layout_old], [items, items_old]):
            if current_layout is None:
                continue
            global_items = set([entry[0].split(':')[0] for entry in tc.default_global()])
            layout_h = QHBoxLayout()
            current_layout.addLayout(layout_h)
            layout_v = None
            for idx, entry in enumerate(items):
                if idx % 11 == 0:
                    if layout_v is not None:
                        layout_v.addStretch(1)
                    else:
                        pass
                    if layout_v is not None:
                        layout_h.addWidget(Separator(typ='vertical', color='white'), stretch=0)
                    layout_v = QVBoxLayout()
                    layout_h.addLayout(layout_v, stretch=1)
                label_name = entry[self.idx_name].split(':')[self.idx_name_name]
                layout_v.addWidget(QLabel(label_name, self), stretch=0)

                # Behaviour based on typ

                if entry[self.idx_type] in ('COMBO', 'COMBOX'):
                    widget = QComboBox(self)
                    widget.addItems(entry[self.idx_values])
                    widget.setCurrentIndex(0)
                    widget.currentIndexChanged.connect(self._change_color_to_changed)
                elif entry[self.idx_type] in ('DIR', 'DIR/SEARCH'):
                    widget = QLineEdit(entry[self.idx_values], self)
                    widget.textChanged.connect(self._change_color_to_changed)
                    widget.returnPressed.connect(self._find_dir)
                    widget.setPlaceholderText('Press shift+return')
                elif entry[self.idx_type] in ('FILE', 'FILE/SEARCH'):
                    widget = QLineEdit(entry[self.idx_values], self)
                    widget.textChanged.connect(self._change_color_to_changed)
                    widget.returnPressed.connect(self._find_file)
                    widget.setPlaceholderText('Press shift+return')
                elif entry[self.idx_type] == 'PLAIN':
                    widget = QLineEdit(entry[self.idx_values], self)
                    widget.textChanged.connect(self._change_color_to_changed)
                elif entry[self.idx_type] == 'PASSWORD':
                    widget = QLineEdit(entry[self.idx_values], self)
                    widget.textChanged.connect(self._change_color_to_changed)
                    widget.setEnabled(False)
                else:
                    raise IOError('{0}: {1} not known!'.format(entry[self.idx_name], entry[self.idx_type]))
                assert label_name not in self.label_dict, (label_name, self.label_dict)
                self.label_dict[label_name] = widget
                widget.setObjectName('setting')

                exclude_typ_list = [
                    'Mount',
                    'Font',
                    'Others',
                    'Notification_widget',
                    ]
                if self.typ not in exclude_typ_list and not entry[self.idx_name].startswith('WIDGETS') and not entry[self.idx_name] == 'IMPORTANT':
                    widget_2 = QComboBox(self)
                    widget_2.addItems(['Main', 'Advanced', 'Rare'])
                    combo_idx = widget_2.findText(entry[self.idx_priority])
                    assert combo_idx >= 0, entry
                    widget_2.setCurrentIndex(combo_idx)
                    widget_2.currentIndexChanged.connect(self._change_color_to_changed)
                else:
                    widget_2 = None

                try:
                    global_name = entry[self.idx_name].split(':')[self.idx_name_global:]
                    if not global_name:
                        raise IndexError
                except IndexError:
                    global_name = None
                    widget_3 = None
                else:
                    for entry_global in global_name:
                        if entry_global not in global_items:
                            assert False, (entry_global, 'not in ', global_items)
                    widget_3 = QPushButton(self)
                    widget_3.setCheckable(True)
                    widget_3.setText('GLOBAL')
                    widget_3.toggled.connect(self._toggle_change)

                widget.setObjectName('default_settings')
                layout_h_2 = QHBoxLayout()
                layout_h_2.setContentsMargins(0, 0, 0, 0)
                layout_h_2.addWidget(widget, stretch=1)
                for test_widget in [widget_2, widget_3]:
                    if test_widget is None:
                        continue
                    else:
                        test_widget.setObjectName('default_settings')
                        layout_h_2.addWidget(test_widget, stretch=0)

                layout_v.addLayout(layout_h_2, stretch=1)
                self.content.append({
                    'widget': widget,
                    'widget_2': widget_2,
                    'widget_3': widget_3,
                    'settings': {
                        'typ': entry[self.idx_type],
                        'name': entry[self.idx_name].split(':')[self.idx_name_name],
                        'name_global': global_name,
                        'values': entry[self.idx_values],
                        'dtype': entry[self.idx_dtype],
                        'group': entry[self.idx_group],
                        'tooltip': entry[self.idx_tooltip],
                        }
                    })

                if widget_3 is not None:
                    widget_3.setChecked(True)
            layout_v.addStretch(1)

    @pyqtSlot(bool)
    def _toggle_change(self, state):
        """
        Change the color of the entry to color changed.

        Arguments:
        None

        Return:
        None
        """
        for entry in self.content:
            if entry['widget_3'] == self.sender():
                entry['widget'].setEnabled(not state)

    @pyqtSlot()
    def _change_color_to_changed(self):
        """
        Change the color of the entry to color changed.

        Arguments:
        None

        Return:
        None
        """
        self.sender().setStyleSheet(tu.get_style(typ='changed'))

    @pyqtSlot()
    def _button_clicked(self):
        """
        Emit the signal, if the delete button is clicked.
        Send the seperator widget for deletion.

        Arguments:
        None

        Return:
        None
        """
        self.delete.emit(self.separator)

    def get_settings(self):
        """
        Get the settings from the child widgets.

        Arguments:
        None

        Return:
        List of settings
        """
        settings_list = []
        error_occured = False
        for entry in self.content:
            widget = entry['widget']
            settings = copy.deepcopy(entry['settings'])
            dtype = settings['dtype']
            key = settings['name']
            default_value = settings['values']
            if isinstance(widget, QComboBox):
                value = widget.currentText()
            elif isinstance(widget, QLineEdit):
                value = widget.text()
            else:
                message = '{0}: Type {1} not known!'.format(key, type(widget))
                print(message)
                tu.message(message)
                sys.exit()

            for number in ['2', '3']:
                temp_widget = entry['widget_{0}'.format(number)]
                if isinstance(temp_widget, QComboBox):
                    settings['widget_{0}'.format(number)] = temp_widget.currentText()
                elif isinstance(temp_widget, QPushButton):
                    settings['widget_{0}'.format(number)] = temp_widget.isChecked()
                else:
                    assert temp_widget is None
                    settings['widget_{0}'.format(number)] = temp_widget

            if value:
                if tu.check_instance(value=value, typ=dtype):
                    widget.setStyleSheet(tu.get_style(typ='unchanged'))
                else:
                    widget.setStyleSheet(tu.get_style(typ='error'))
                    message = '{0}: {1} needs to be {2}'.format(
                        key,
                        value,
                        dtype
                        )
                    print(message)
                    tu.message(message)
                    error_occured = True
            else:
                if default_value:
                    widget.setStyleSheet(tu.get_style(typ='unchanged'))
                    #print('WARNING: {0} is empty!'.format(key))
                else:
                    pass

            if key == 'Typ' and not widget.isEnabled():
                value = 'Copy_to_hdd'
            elif key == 'Protocol' and not widget.isEnabled():
                value = 'hdd'
            else:
                pass
            settings_list.append({key: [value, settings]})

        if error_occured:
            return None
        else:
            return settings_list

    def set_settings(self, settings):
        """
        Set settings to the entry widgets.

        Arguments:
        settings - List of settings

        Return:
        None
        """
        disable_name = False
        disable_typ = False
        for entry in settings:
            if self.typ == 'Mount':
                if 'Mount name' in entry:
                    if entry['Mount name'][0] == 'HDD':
                        disable_name = True
                    else:
                        pass
                elif 'Typ' in entry:
                    if entry['Typ'][0] == 'Copy_to_hdd':
                        disable_typ = True
                    else:
                        pass
                else:
                    pass
            else:
                pass

            if disable_name and disable_typ:
                is_hdd = True
                self.setEnabled(False)
            else:
                is_hdd = False
                self.setEnabled(True)

            for key in entry:
                idx = None
                for idx, setting in enumerate(self.content):
                    name = setting['settings']['name']
                    if key != name:
                        continue
                    else:
                        break

                if idx is None:
                    continue
                else:
                    pass

                if name == key:
                    pass
                else:
                    continue

                try:
                    widget = self.content[idx]['widget']
                except KeyError:
                    continue
                else:
                    pass

                if isinstance(widget, QComboBox):
                    hdd_idx = widget.findText(entry[key][0])
                    if idx < 0:
                        if is_hdd:
                            widget.addItem('Copy_to_hdd')
                            hdd_idx = widget.findText('Copy_to_hdd')
                        else:
                            hdd_idx = 0
                    else:
                        pass
                    widget.setCurrentIndex(hdd_idx)
                elif isinstance(widget, QLineEdit):
                    widget.setText(entry[key][0])
                else:
                    print(key, type(widget, 'Not known!'))
                    sys.exit()
                widget.setStyleSheet(tu.get_style(typ='unchanged'))

                try:
                    widget_2 = self.content[idx]['widget_2']
                except KeyError:
                    continue
                else:
                    pass

                if isinstance(widget_2, QComboBox):
                    try:
                        widget_2_idx = widget_2.findText(entry[key][1]['widget_2'])
                    except KeyError:
                        print(
                            'Older version of a save file detected!',
                            'Please save default settings again!'
                            )
                        continue
                    else:
                        widget_2_idx = max(0, widget_2_idx)
                        widget_2.setCurrentIndex(widget_2_idx)
                        widget_2.setStyleSheet(tu.get_style(typ='unchanged'))
                elif widget_2 is None:
                    pass
                else:
                    pass

                try:
                    widget_3 = self.content[idx]['widget_3']
                except KeyError:
                    continue
                else:
                    pass

                if isinstance(widget_3, QPushButton):
                    try:
                        widget_3.setChecked(entry[key][1]['widget_3'])
                    except KeyError:
                        widget_3.setChecked(True)
                    except TypeError:
                        widget_3.setChecked(True)
                elif widget_3 is None:
                    pass
                else:
                    pass

    def _find_label(self, widget):
        for key, value in self.label_dict.items():
            if value == widget:
                return key

    @pyqtSlot()
    def _find_file(self):
        """
        Find file name to insert it into the widget.

        Arguments:
        None

        Return:
        None
        """
        in_file = QFileDialog.getOpenFileName(
            caption='Find file: {0} - {1}'.format(self.typ, self._find_label(self.sender())),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )

        in_file = in_file[0]

        if in_file != '':
            self.sender().setText(in_file)

    @pyqtSlot()
    def _find_dir(self):
        """
        Find directory to insert it into the widget.

        Arguments:
        None

        Return:
        None
        """
        in_dir = QFileDialog.getExistingDirectory(
            caption='Find directory: {0} - {1}'.format(self.typ, self._find_label(self.sender())),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )
        if in_dir != '':
            self.sender().setText(in_dir)

from .separator import Separator



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import urllib3
import socket
import smtplib
from email.mime.text import MIMEText
import telepot
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton, QHBoxLayout
from PyQt5.QtCore import pyqtSlot, pyqtSignal

from . import transphire_utils as tu

class NotificationContainer(QWidget):
    """
    Container for notification widgets

    Inherits:
    QWidget

    Signals:
    sig_stop - Signal is emitted, if the user is sending /stop via telegram
    """
    sig_stop = pyqtSignal()

    def __init__(
            self, content, mount_worker, process_worker,
            settings_folder, parent=None, **kwargs
            ):
        """
        Initialise layout

        Arguments:
        content - TranSPHIRE content
        mount_worker - Mount worker object
        process_worker - Process worker object
        settings_folder - Folder to load/save settings from/to
        parent - Parent widget (default None)

        Returns:
        None
        """
        super(NotificationContainer, self).__init__(parent)

        noti_content = {}
        for entry in content:
            for widget in entry:
                for key in widget:
                    noti_content[key] = widget[key][0]
        bot_token = noti_content['Bot token'].strip()
        self.smtp_server = noti_content['SMTP server']
        self.sending_email = noti_content['E-Mail adress']

        # Global content
        self.content = {}
        self.users_telegram = {}
        self.users_email = {}
        self.block_list = []
        self.bot = telepot.Bot(bot_token)
        self.parent = parent
        self.settings_folder = settings_folder
        self.update_id = 0
        self.enable_telegram = None
        exception_list = []

        # Layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        max_per_col = 3

        default_programs_dict = {
            'telegram': 'T',
            'email': '@'
            }

        # Add to layout
        for key, value  in default_programs_dict.items():
            layout_default = QHBoxLayout()
            layout.addLayout(layout_default)
            if noti_content['Default names {0}'.format(key)]:
                offset = 0
                for idx, entry in enumerate(noti_content['Default names {0}'.format(key)].split(';')):
                    try:
                        name, default = entry.split(':')
                    except ValueError:
                        print('Default {0} entry does not follow - Name:{0} Name - {1}'.format(key, entry))
                        offset += 1
                        continue
                    if idx-offset % max_per_col == 0 and idx != 0:
                        layout_default.addStretch(1)
                        layout_default = QHBoxLayout()
                        layout.addLayout(layout_default)
                    name = '{0} {1}'.format(value, name)
                    default = '{0} {1}'.format(value, default)
                    exception_list.append(default)
                    widget = NotificationWidget(name=name, default=default, parent=self, default_programs_dict=default_programs_dict)
                    layout_default.addWidget(widget)
                    self.content[name] = widget
                if noti_content['Default names {0}'.format(key)].split(';'):
                    layout_default.addStretch(1)
            else:
                pass

        # Chooseable users
        layout_default_user = QHBoxLayout()
        layout.addLayout(layout_default_user)
        for idx in range(int(noti_content['Number of users'])):
            if idx % max_per_col == 0 and idx != 0:
                layout_default_user.addStretch(1)
                layout_default_user = QHBoxLayout()
                layout.addLayout(layout_default_user)
            name = 'User {0}'.format(idx)
            default = 'choose'
            widget = NotificationWidget(name=name, default=default, parent=self, default_programs_dict=default_programs_dict)
            for entry in exception_list:
                widget.add_exceptions(name=entry)
            layout_default_user.addWidget(widget)
            self.content[name] = widget
        if int(noti_content['Number of users']) != 0:
            layout_default_user.addStretch(1)

        # Test button
        layout_button = QHBoxLayout()
        layout.addLayout(layout_button)
        self.button_telegram = QPushButton('Update Telegram', self)
        self.button_telegram.setToolTip('Update Telegram')
        self.button_email = QPushButton('Add E-Mail', self)
        self.button_email.setToolTip('Add E-Mail')
        self.button_test = QPushButton('Test', self)
        self.button_test.setToolTip('Test')
        self.button_telegram.setObjectName('notification')
        self.button_test.setObjectName('notification')
        self.button_email.setObjectName('notification')
        layout_button.addWidget(self.button_telegram)
        layout_button.addWidget(self.button_email)
        layout_button.addWidget(self.button_test)
        layout_button.addStretch(1)

        layout.addStretch(1)

        # Events
        self.button_email.clicked.connect(self.add_email)
        self.button_telegram.clicked.connect(self.update)
        self.button_test.clicked.connect(lambda: self.send_notification('Test successful'))
        mount_worker.sig_notification.connect(self.send_notification)
        process_worker.sig_notification.connect(self.send_notification)

        self.enable_telegram = True
        self.enable_email = True
        self.first = True

    def get_settings(self):
        """
        Get settings as dict

        Arguments:
        None

        Returns:
        Settings dictionary as list
        """
        settings = {}
        for key in self.content:
            settings.update(self.content[key].get_settings())
        return [settings]

    def set_settings(self, settings):
        """
        Set settings.

        Arguments:
        settings - Settings as dictionary to set.

        Returns:
        None
        """
        for key in settings:
            try:
                self.content[key].set_settings(*settings[key])
            except KeyError:
                #print('Key', key, 'no longer exists')
                continue

    @pyqtSlot()
    def update(self):
        """
        Update the combo boxes.

        Arguments:
        None

        Returns:
        None
        """
        for key in self.content:
            self.content[key].clear_combo()
        if self.enable_telegram:
            self.update_telegram()
        else:
            pass
        if self.enable_email:
            self.update_email()
        else:
            pass
        if self.first:
            self.first = False
        else:
            pass

    @pyqtSlot()
    def add_email(self):
        """
        Add a new e-mail adress

        Arguments:
        None

        Returns:
        None
        """
        email_dialog = EmailDialog(self)
        result = email_dialog.exec_()
        if result:
            save_file = os.path.join(
                self.settings_folder,
                '.users_email'
                )
            user = email_dialog.get_name()
            email = email_dialog.get_email()
            self.users_email[user] = email
            with open(save_file, 'w') as write:
                for key in self.users_email:
                    write.write('{0}\t{1}\n'.format(key, self.users_email[key]))
        else:
            pass
        self.update()

    def update_email(self):
        """
        Update E-Mail file.

        Arguments:
        None

        Returns:
        None
        """
        save_file = os.path.join(
            self.settings_folder,
            '.users_email'
            )

        # Check if a save file exists
        if os.path.exists(save_file):
            with open(save_file) as read:
                for line in read:
                    name, email = line.replace('\n', '').split('\t')
                    self.users_email[name] = email
        for key in self.content:
            self.content[key].update_combo('email', self.users_email)

        try:
            server = smtplib.SMTP(self.smtp_server, timeout=10)
        except socket.timeout:
            self.enable_email = False
            print('EMail SMTP server adress not acessable!')
            return None
        except socket.gaierror:
            self.enable_email = False
            print('EMail SMTP server adress wrong or empty')
            return None
        except smtplib.SMTPServerDisconnected:
            self.enable_email = False
            print('EMail SMTP server adress wrong or empty')
            return None
        else:
            try:
                server.quit()
            except smtplib.SMTPServerDisconnected:
                self.enable_email = False
                print('EMail SMTP server adress wrong or empty')
                return None

    def update_telegram(self):
        """
        Update telegram settings file

        Arguments:
        None

        Returns:
        None
        """
        save_file = os.path.join(
            self.settings_folder,
            '.users_telegram'
            )

        # Check if a save file exists
        if os.path.exists(save_file):
            with open(save_file) as read:
                for line in read:
                    name, user_id = line.replace('\n', '').split('\t')
                    self.users_telegram[name] = user_id

        try:
            response = self.bot.getUpdates()
        except telepot.exception.TelegramError:
            self.enable_telegram = False
            print('Telegram bot token wrong or empty!')
            return None
        except urllib3.exceptions.MaxRetryError:
            self.enable_telegram = False
            print('Telegram bot not acessable!')
            return None
        except telepot.exception.BadHTTPResponse:
            self.enable_telegram = False
            print('Telegram bot not acessable!')
            return None
        self.users_telegram.update(self.get_telegram_user(response))

        with open(save_file, 'w') as write:
            for key in list(self.users_telegram.keys()):
                try:
                    write.write('{0}\t{1}\n'.format(key, self.users_telegram[key]))
                except UnicodeEncodeError:
                    print('Found a non-unicode character, that cannot be decoded.')
                    del self.users_telegram[key]

        for key in self.content:
            self.content[key].update_combo('telegram', self.users_telegram)

    @pyqtSlot(str)
    def send_notification(self, text):
        """
        Send a notification to specified users.

        Arguments:
        text - Text to send

        Returns:
        None
        """

        text = '{0}:\n{1}'.format(os.uname()[1], text)
        print(text)
        chosen_users = self.get_settings()
        for entry in chosen_users:
            for key in entry:
                name, state = entry[key].split('\t')

                if self.enable_telegram:
                    telegram_name = name[2:]

                    if telegram_name in self.users_telegram and \
                            state == 'True' and \
                            name.startswith('T '):
                        user_id = self.users_telegram[telegram_name]
                        try:
                            self.bot.sendMessage(user_id, text)
                        except telepot.exception.BotWasBlockedError as err:
                            if user_id in self.block_list:
                                pass
                            else:
                                self.block_list.append(user_id)
                                tu.message('{0}\n{1}\t{2}'.format(
                                    os.uname()[1],
                                    name,
                                    err
                                    ))
                        except telepot.exception.TooManyRequestsError:
                            print('Could not send: {}'.format(text))
                        except telepot.exception.TelegramError as err:
                            if user_id in self.block_list:
                                pass
                            else:
                                self.block_list.append(user_id)
                                tu.message('{0}\n{1}\t{2}'.format(
                                    os.uname()[1],
                                    name,
                                    err
                                    ))
                        else:
                            if user_id in self.block_list:
                                self.block_list.remove(user_id)
                            else:
                                pass

                    else:
                        pass

                else:
                    pass

                if self.enable_email:
                    email_name = name[2:]

                    if email_name in self.users_email and \
                            state == 'True' and \
                            name.startswith('@ '):
                        email = self.users_email[email_name]
                        msg = MIMEText(text)
                        msg['Subject'] = 'Transphire message {0}'.format(
                            os.uname()[1]
                            )
                        msg['From'] = self.sending_email
                        msg['To'] = email
                        try:
                            server = smtplib.SMTP(self.smtp_server, timeout=10)
                        except smtplib.SMTPServerDisconnected:
                            print('Could not send to:', name)
                            continue
                        except Exception:
                            print('Unknown exception! Could not send to:', name)
                            continue

                        try:
                            server.sendmail(self.sending_email, [email], msg.as_string())
                        except smtplib.SMTPServerDisconnected:
                            print('Could not send to:', name)
                            continue
                        except smtplib.SMTPRecipientsRefused:
                            print('Could not send to:', name)
                            continue
                        except Exception:
                            print('Unknown exception! Could not send to:', name)
                            continue

                        try:
                            server.quit()
                        except smtplib.SMTPServerDisconnected:
                            print('Could not send to:', name)
                            continue
                        except Exception:
                            print('Unknown exception! Could not send to:', name)
                            continue
                    else:
                        pass

                else:
                    pass

        else:
            pass

    def get_telegram_user(self, page):
        """
        Get the users and the user id

        Arguments:
        page - Webpage of the telegram bot

        Returns:
        Dictionary of users
        """
        user_dict = {}
        for entry in page:
            msg = entry['message']
            try:
                msg['text']
            except KeyError:
                continue
            if msg['text'] == '/start' or msg['text'] == '/refresh':
                user_id = None
                first_name = None
                last_name = None
                for key in msg['chat']:
                    if key == 'id':
                        user_id = msg['chat']['id']
                    elif key == 'first_name':
                        first_name = msg['chat']['first_name']
                    elif key == 'last_name':
                        last_name = msg['chat']['last_name']
                if last_name is None:
                    key = '{0}'.format(first_name)
                else:
                    key = '{0} {1}'.format(first_name, last_name)
                user_dict[key] = user_id
            else:
                pass
            if self.first:
                self.update_id = entry['update_id'] + 1
            else:
                pass
        return user_dict

    def enable(self, var, use_all):
        """
        Enable or disable the widgets of the widget.

        Arguments:
        var - True(Enable) or False(Disable)
        use_all - Disable or enable all

        Returns:
        None
        """
        for key in self.content:
            if use_all:
                self.content[key].setEnabled(var)
                self.button_email.setEnabled(var)
                self.button_telegram.setEnabled(var)
                self.button_test.setEnabled(var)
            else:
                pass

    def get_telegram_messages(self):
        """
        Check the messages in telegram for commands.

        Arguments:
        None

        Returns:
        None
        """
        if not self.enable_telegram:
            return None
        else:
            pass

        try:
            page_dict = self.bot.getUpdates(offset=self.update_id)
        except telepot.exception.TelegramError:
            print('Telegram bot token wrong or empty!')
            return None
        except telepot.exception.BadHTTPResponse:
            print('Telegram bot not acessable!')
            return None
        except urllib3.exceptions.MaxRetryError:
            print('Telegram bot not acessable!')
            return None
        except urllib3.exceptions.ReadTimeoutError:
            print('Telegram bot not acessable!')
            return None
        except Exception as e:
            print(e)
            return None

        for line in page_dict:
            try:
                msg = line['message']
                self.update_id = line['update_id'] + 1
                try:
                    first_name = msg['from']['first_name']
                except KeyError:
                    first_name = None
                try:
                    last_name = msg['from']['last_name']
                except KeyError:
                    last_name = None
                if first_name is None and last_name is None:
                    user = 'Anonymus User'
                elif first_name is None:
                    user = last_name
                elif last_name is None:
                    user = first_name
                else:
                    user = '{0} {1}'.format(first_name, last_name)

                if msg['text'].startswith('/send'):
                    text = msg['text'][len('/send'):]
                    self.send_notification('{0}: {1}'.format(user, text))
                elif msg['text'].startswith('/stop'):
                    self.send_notification('Stopping {0}'.format(os.uname()[1]))
                    self.sig_stop.emit()
                else:
                    pass
            except Exception:
                continue

    def send_to_user(self, user_id, text, name):
        """
        Send a message to all users.

        Arguments:
        user_id - User id of the receiving person
        text - Text of the message
        name - Name of the sender

        Returns:
        None
        """
        try:
            self.bot.sendMessage(
                user_id,
                '{0}\n{1}: {2}'.format(os.uname()[1], name, text)
                )
        except telepot.exception.BotWasBlockedError as err:
            tu.message('{0}\n{1}\t{2}'.format(
                os.uname()[1],
                name,
                err
                ))

from .notificationwidget import NotificationWidget
from .emaildialog import EmailDialog



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QTabWidget, QTabBar, QStylePainter, QStyleOptionTab, QStyle
from PyQt5.QtCore import QPoint, QRect, pyqtSlot, pyqtSignal


class MyTabBar(QTabBar):

    def __init__(self, parent):
        super(MyTabBar, self).__init__(parent)

    def tabSizeHint(self, index):
        s = super(MyTabBar, self).tabSizeHint(index)
        s.transpose()
        return s

    def paintEvent(self, event):
        painter = QStylePainter(self)
        opt = QStyleOptionTab()

        for i in range(self.count()):
            self.initStyleOption(opt, i)
            painter.drawControl(QStyle.CE_TabBarTabShape, opt)
            painter.save()

            s = opt.rect.size()
            s.transpose()
            r = QRect(QPoint(), s)
            r.moveCenter(opt.rect.center())
            opt.rect = r

            c = self.tabRect(i).center()
            painter.translate(c)
            painter.rotate(90)
            painter.translate(-c)
            painter.drawControl(QStyle.CE_TabBarTabLabel, opt)
            painter.restore()

class TabDocker(QWidget):
    """
    Tab widget for the settingswidgets.

    Inherits:
    QWidget
    """
    sig_start_plot = pyqtSignal()
    latest_active = [None]

    def __init__(self, parent=None, **kwargs):
        """
        Initialise layout for TabDocker

        Arguments:
        parent - Parent widget (default None)

        Return:
        None
        """
        super(TabDocker, self).__init__(parent)
        self.parent = parent
        try:
            self.layout = kwargs['layout']
        except KeyError:
            self.layout = None
        try:
            self.name = kwargs['name']
        except KeyError:
            self.name = None
        self.widgets = []

        layout_tmp = QVBoxLayout(self)
        self.parent_widget = QWidget(self)
        layout_tmp.addWidget(self.parent_widget)
        layout_tmp.setContentsMargins(0, 0, 0, 0)

        layout = QVBoxLayout(self.parent_widget)
        layout.setContentsMargins(0, 0, 0, 0)
        self.tab_widget = QTabWidget(self)
        if self.layout in ('TAB1', 'Settings'):
            tab_bar = MyTabBar(self.tab_widget)
            tab_bar.setObjectName('vertical')
            self.tab_widget.setObjectName('vertical')
            self.tab_widget.setTabBar(tab_bar)
            self.tab_widget.setTabPosition(QTabWidget.West)
        layout.addWidget(self.tab_widget)

        self.tab_widget.currentChanged.connect(self.assign_latest)

    @pyqtSlot(int)
    def assign_latest(self, idx):
        current_name = self.tab_widget.tabText(idx)
        try:
            parent_content = self.parent.content[self.layout].name
        except AttributeError: # Exception for the Default settings dialog
            parent_content = False
        except TypeError: # Exception for the Default settings dialog
            parent_content = False
        except KeyError: # Exception for the main window dialog
            parent_content = False

        check_list = (parent_content, self.name, current_name)

        latest_active = self
        for list_idx, entry in enumerate(check_list):
            if entry == 'Visualisation':
                cur_tab_widget = self.tab_widget.widget(idx)
                try:
                    for i in range(list_idx):
                        idx = cur_tab_widget.currentIndex()
                        cur_tab_widget = cur_tab_widget.widget(idx)
                    latest_active = cur_tab_widget if cur_tab_widget is not None else self
                except:
                    latest_active = self
                break
        if self.latest_active[0] != latest_active:
            self.latest_active[0] = latest_active
            latest_active.sig_start_plot.emit()

    def setCurrentIndex(self, idx):
        """
        Set the current Index of the tab_widget.

        Arguments:
        idx - Index to set

        Returns: Current index of self.tab_widget
        """
        return self.tab_widget.setCurrentIndex(idx)

    def setCurrentWidget(self, widget):
        """
        Set the current widget of the tab_widget.

        Arguments:
        idx - Widget to set

        Returns: Current index of self.tab_widget
        """
        return self.tab_widget.setCurrentWidget(widget)

    def currentIndex(self):
        """
        Get the current Index of the tab_widget.

        Returns: Current index of self.tab_widget
        """
        return self.tab_widget.currentIndex()

    def add_tab(self, widget, name, add_widgets=True):
        """
        Add a new tab to the TabDocker

        Arguments:
        widget - Widget to add
        name - Name of the widget

        Return:
        None
        """
        if isinstance(widget, TabDocker):
            widget.parent_widget.setObjectName('tab')
        else:
            pass
        current_state = self.tab_widget.blockSignals(True)
        index = self.tab_widget.addTab(widget, name)
        if add_widgets:
            self.widgets.append(widget)
        self.tab_widget.blockSignals(current_state)
        self.tab_widget.setTabToolTip(index, name)

    def count(self):
        """
        Return the number of tabs.

        Arguments:
        None

        Returns:
        Number of tabs
        """
        return self.tab_widget.count()

    def widget(self, idx):
        """
        Return the widget that belongs to the idx of tabs.

        Arguments:
        idx - Tab index

        Returns:
        Widget
        """
        return self.tab_widget.widget(idx)

    def setMovable(self, status):
        """
        Set the movable status for the tab widgets

        Arguments:
        status - Boolean variable for the status

        Returns:
        None
        """
        return self.tab_widget.setMovable(status)

    def tabText(self, idx):
        """
        Return the text of the tab at idx

        Arguments:
        idx - Index of the tab

        Returns:
        Text of the tab at position isx
        """
        return self.tab_widget.tabText(idx)

    def setTabText(self, idx, text):
        """
        Set the text for the tab at idx

        Arguments:
        idx - Index of the tab
        text - Text of the tab

        Returns:
        None
        """
        return self.tab_widget.setTabText(idx, text)

    def removeTab(self, idx):
        """
        Remove the widget located at tab idx

        Arguments:
        idx - Idx of the widget

        Returns:
        None
        """
        current_state = self.tab_widget.blockSignals(True)
        idx = self.tab_widget.removeTab(idx)
        self.tab_widget.blockSignals(current_state)
        return idx

    def indexOf(self, widget):
        """
        Get the index of the widget.

        Arguments:
        widget - Adress of the widget

        Returns:
        Index of the widget
        """
        return self.tab_widget.indexOf(widget)

    def setTabPosition(self, position):
        """
        Set the tab position of the Tab bar

        Arguments:
        position - Tab position as string ['North', 'East', 'West', 'South']

        Returns:
        None
        """
        tab_position_dict = {
            'North': QTabWidget.North,
            'South': QTabWidget.South,
            'West': QTabWidget.West,
            'East': QTabWidget.East,
            }
        self.tab_widget.setTabPosition(tab_position_dict[position])

    def setTabEnabled(self, index, state):
        """
        Set the tab position index to the enable state.

        Arguments:
        index - Tab position index
        state - State (True or False)

        Returns:
        None
        """
        self.tab_widget.setTabEnabled(index, state)

    def order_tabs(self):
        current_state = self.tab_widget.blockSignals(True)
        widget_tuple = tuple([
            (
                self.widget(idx).name,
                self.widget(idx),
                self.tab_widget.isTabEnabled(idx)
                )
            for idx in range(self.count())
            ])
        for idx in reversed(range(self.count())):
            self.removeTab(idx)

        for idx, (name, widget, state) in enumerate(sorted(widget_tuple)):
            self.add_tab(widget, name, add_widgets=False)
            self.setTabEnabled(idx, state)
            if state:
                self.setCurrentIndex(idx)
        self.tab_widget.blockSignals(current_state)

    def enable_tab(self, visible):
        """
        Enable or disable the tab.


        Arguments:
        visible - Enable if True, Disable if False
        name - Name of the tab to disable.

        Returns:
        None
        """
        index = self.parent.content[self.layout].indexOf(self)
        if not visible:
            self.parent.content[self.layout].removeTab(index)
        else:
            self.parent.content[self.layout].add_tab(self, self.name)
            self.parent.content[self.layout].order_tabs()




import sys
import chimerax.core.commands as ccc

print(sys.argv)
input_file = sys.argv[1]
output_file = sys.argv[2]

rotation_list = [
    [0, 0],
    [0, 90],
    [90, 90],
    [180, 0]
    ]

for list_index in range(4):
    volume_index = list_index + 1
    ccc.run(session, 'open {}'.format(input_file))
    ccc.run(session, 'volume #{} step 1'.format(volume_index))
    ccc.run(session, 'volume #{} sdLevel 5'.format(volume_index))
    ccc.run(session, 'turn z {} models #{}'.format(rotation_list[list_index][0], volume_index))
    ccc.run(session, 'turn x {} models #{}'.format(rotation_list[list_index][1], volume_index))

ccc.run(session, 'windowsize 800 800')
ccc.run(session, 'set bgColor white')
ccc.run(session, 'tile')
ccc.run(session, 'save {}'.format(output_file))
ccc.run(session, 'exit')



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import json
import os
import imageio
import numpy as np

from . import transphire_utils as tu


def get_picking_command(file_input, new_name, settings, queue_com, name):
    """
    Create the picking command based on the picking software.

    file_input - Input name of the file for ctf estimation
    new_name - Output file
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    Picking command
    File to check vor validation if the process was successful
    """
    picking_name = settings['Copy']['Picking']
    command = None
    block_gpu = None
    gpu_list = None
    if tu.is_higher_version(picking_name, '1.4.1'):
        command, gpu = create_cryolo_v1_4_1_command(
            picking_name=picking_name,
            file_input=file_input,
            file_output=new_name,
            settings=settings,
            name=name,
            )
        check_files = []
        if '_' in gpu:
            if float(settings[picking_name]['--gpu_fraction']) == 1:
                raise UserWarning('Sub GPUs are only supported if the --gpu_fraction option is not equal 1')
            block_gpu = True
        else:
            block_gpu = True
        gpu_list = gpu.split()

    elif tu.is_higher_version(picking_name, '1.1'):
        command, gpu = create_cryolo_v1_1_0_command(
            picking_name=picking_name,
            file_input=file_input,
            file_output=new_name,
            settings=settings,
            name=name,
            )
        if '_' in gpu:
            raise UserWarning('Sub GPUs are only supported in crYOLO version >=1.4.1')
        check_files = []
        block_gpu = True
        gpu_list = gpu.split()

    elif tu.is_higher_version(picking_name, '1.0'):
        command, gpu = create_cryolo_v1_0_4_command(
            picking_name=picking_name,
            file_input=file_input,
            file_output=new_name,
            settings=settings,
            name=name,
            )
        if '_' in gpu:
            raise UserWarning('Sub GPUs are only supported in crYOLO version >=1.4.1')
        check_files = []
        block_gpu = True
        gpu_list = gpu.split()

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['Picking']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        IOError(message)

    assert command is not None, 'command not specified: {0}'.format(picking_name)
    assert block_gpu is not None, 'block_gpu not specified: {0}'.format(picking_name)
    assert gpu_list is not None, 'gpu_list not specified: {0}'.format(picking_name)

    return command, check_files, block_gpu, gpu_list


def find_logfiles(root_path, file_name, settings, queue_com, name):
    """
    Find logfiles related to the produced CTF files.

    root_path - Root path of the file
    file_name - File name of the ctf file.
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    list of log files
    """
    folder_name = 'picking_folder_feedback_{0}'.format(settings['do_feedback_loop'].value)
    log_files = None
    copied_log_files = None
    picking_name = settings['Copy']['Picking']
    picking_root_path = os.path.join(settings[folder_name], file_name)
    if tu.is_higher_version(picking_name, '1.4.1'):
        if settings[picking_name]['--filament'] == 'True':
            folder_names = (
                ('EMAN_HELIX_SEGMENTED', 'box'),
                ('EMAN_START_END', 'box'),
                ('STAR_START_END', 'star'),
                )
        else:
            folder_names = (
                ('CBOX', 'cbox'),
                ('EMAN', 'box'),
                ('STAR', 'star'),
                )
        copied_log_files = []
        for folder_name, ext in folder_names:
            copied_log_files.extend(['{0}.{1}'.format(os.path.join(
                os.path.dirname(picking_root_path),
                folder_name,
                os.path.basename(picking_root_path)
                ),
                ext,
                )])
        log_files = copied_log_files
    elif tu.is_higher_version(picking_name, '1.2.2'):
        if settings[picking_name]['--filament'] == 'True':
            folder_names = (
                ('EMAN_HELIX_SEGMENTED', 'box'),
                ('EMAN_START_END', 'box'),
                ('STAR_START_END', 'star'),
                )
        else:
            folder_names = (
                ('EMAN', 'box'),
                ('STAR', 'star'),
                )
        copied_log_files = []
        for folder_name, ext in folder_names:
            copied_log_files.extend(['{0}.{1}'.format(os.path.join(
                os.path.dirname(picking_root_path),
                folder_name,
                os.path.basename(picking_root_path)
                ),
                ext,
                )])
        log_files = copied_log_files
    elif tu.is_higher_version(picking_name, '1.2.1'):
        copied_log_files = ['{0}.box'.format(picking_root_path)]
        log_files = copied_log_files

    elif tu.is_higher_version(picking_name, '1.1.0'):
        if settings[picking_name]['--filament'] == 'True':
            copied_log_files = ['{0}.txt'.format(picking_root_path)]
        else:
            copied_log_files = ['{0}.box'.format(picking_root_path)]
        log_files = copied_log_files

    elif tu.is_higher_version(picking_name, '1.0.4'):
        copied_log_files = ['{0}.box'.format(picking_root_path)]
        log_files = copied_log_files

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['Picking']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        raise IOError(message)

    assert log_files is not None
    assert copied_log_files is not None
    return log_files, copied_log_files


def create_filter_command(
        file_input, settings
        ):
    picking_name = settings['Copy']['Picking']
    command = []
    block_gpu = False
    gpu_list = []

    folder_name = 'picking_folder_feedback_{0}'.format(settings['do_feedback_loop'].value)

    file_output_tmp = file_input
    try:
        do_filter = bool(settings[picking_name]['Filter micrographs'] == 'True')
    except KeyError:
        do_filter = False
    if do_filter:

        filter_high = float(settings[picking_name]['Filter value high pass (A)'])
        filter_low = float(settings[picking_name]['Filter value low pass (A)'])
        pixel_size = float(settings[picking_name]['Pixel size (A/px)'])

        filter_high_abs = pixel_size / filter_high
        filter_low_abs = pixel_size / filter_low

        file_output_tmp = os.path.join(
                settings[folder_name],
                os.path.basename(file_input)
                )

        command.append("PATH=$(dirname $(which {0})):${{PATH}}".format(settings['Path']['e2proc2d.py']))
        command.append('{0}'.format(settings['Path']['e2proc2d.py']))
        command.append('{0}'.format(file_input))
        command.append('{0}'.format(file_output_tmp))
        command.append('--process=filter.lowpass.gauss:cutoff_freq={0}'.format(filter_low_abs))
        command.append('--process=filter.highpass.gauss:cutoff_freq={0}'.format(filter_high_abs))
        command.append(';')

    file_output_jpg = os.path.join(
            settings[folder_name],
            '{0}.png'.format(os.path.splitext(os.path.basename(file_input))[0])
            )
    command.append("PATH=$(dirname $(which {0})):${{PATH}}".format(settings['Path']['e2proc2d.py']))
    command.append('{0}'.format(settings['Path']['e2proc2d.py']))
    command.append('{0}'.format(file_output_tmp))
    command.append('{0}'.format(file_output_jpg))
    command.append('--meanshrink=4')
    command.append('--outmode=uint8')

    check_files = [file_output_tmp, file_output_jpg]
    return ' '.join(command), file_output_tmp, check_files, block_gpu, gpu_list


def create_cryolo_v1_4_1_command(
        picking_name, file_input, file_output, settings, name
        ):
    """Create the crYOLO v1.1.0 command"""
    command = []
    # Start the program
    ignore_list = []
    ignore_list.append('--filament')
    ignore_list.append('Filter micrographs')
    ignore_list.append('Filter value high pass (A)')
    ignore_list.append('Filter value low pass (A)')
    ignore_list.append('Pixel size (A/px)')
    ignore_list.append('Box size')
    ignore_list.append('Split Gpu?')
    ignore_list.append('--gpu')
    ignore_list.append('--threshold')
    ignore_list.append('--threshold_old')
    ignore_list.append('--weights_old')
    ignore_list.append('Lowest defocus percent')
    ignore_list.append('Minimum micrographs')
    ignore_list.append('Minimum particles')
    ignore_list.append('Mean percent')

    command.append('{0}'.format(settings['Path'][picking_name]))

    command.append('-i')
    command.append('{0}'.format(' '.join(file_input)))
    command.append('-o')
    command.append('{0}'.format(file_output))
    command.append('--write_empty')
    command.append('--threshold={0}'.format(
        settings[picking_name]['--threshold']
        if float(settings[picking_name]['--threshold']) != -1
        else 0.1
        ))

    ignore_list.append('--nomerging')
    ignore_list.append('--nosplit')
    if settings[picking_name]['--filament'] == 'True':
        command.append('--filament')
        if settings[picking_name]['--nomerging'] == 'True':
            command.append('--nomerging')
        if settings[picking_name]['--nosplit'] == 'True':
            command.append('--nosplit')
    else:
        ignore_list.append('--filament_width')
        ignore_list.append('--box_distance')
        ignore_list.append('--minimum_number_boxes')
        ignore_list.append('--mask_width')
        ignore_list.append('--search_range_factor')

    if settings[picking_name]['Split Gpu?'] == 'True':
        try:
            gpu_id = int(name.split('_')[-1])
        except ValueError:
            gpu_id = 0
        try:
            gpu_raw = settings[picking_name]['--gpu'].split()[gpu_id]
        except IndexError:
            raise UserWarning('There are less gpus provided than threads available! Please restart with the same number of pipeline processors as GPUs provided and restart! Stopping this thread!')
    else:
        gpu_raw = settings[picking_name]['--gpu']

    gpu = ' '.join(list(set([entry.split('_')[0] for entry in gpu_raw.split()])))
    if len(gpu.split()) != len(gpu_raw.split()) and settings[picking_name]['Split Gpu?'] == 'False':
        raise UserWarning('One cannot use multi GPU in combination with the disabled Split GPU option!')

    command.append('--gpu')
    command.append('{0}'.format(gpu))

    for key, value in settings[picking_name].items():
        if key in ignore_list:
            continue
        elif value:
            try:
                if '|||' in value:
                    external_log, local_key = value.split('|||')
                    with open(settings[external_log], 'r') as read:
                        log_data = json.load(read)
                    
                    try:
                        set_value = log_data[settings['current_set']][picking_name][local_key]['new_file']
                    except KeyError:
                        continue
                else:
                    set_value = value
            except TypeError:
                set_value = value
            command.append(key)
            command.append('{0}'.format(set_value))

    return ' '.join(command), gpu_raw


def create_cryolo_v1_1_0_command(
        picking_name, file_input, file_output, settings, name
        ):
    """Create the crYOLO v1.1.0 command"""

    command = []
    # Start the program
    ignore_list = []
    ignore_list.append('--filament')
    ignore_list.append('Filter micrographs')
    ignore_list.append('Filter value high pass (A)')
    ignore_list.append('Filter value low pass (A)')
    ignore_list.append('Pixel size (A/px)')
    ignore_list.append('Box size')
    ignore_list.append('Split Gpu?')
    ignore_list.append('--gpu')

    command.append('{0}'.format(settings['Path'][picking_name]))

    command.append('-i')
    command.append('{0}'.format(' '.join(file_input)))
    command.append('-o')
    command.append('{0}'.format(file_output))
    command.append('--write_empty')

    if settings[picking_name]['--filament'] == 'True':
        command.append('--filament')
    else:
        ignore_list.append('--filament_width')
        ignore_list.append('--box_distance')
        ignore_list.append('--minimum_number_boxes')

    if settings[picking_name]['Split Gpu?'] == 'True':
        try:
            gpu_id = int(name.split('_')[-1])-1
        except ValueError:
            gpu_id = 0
        try:
            gpu = settings[picking_name]['--gpu'].split()[gpu_id]
        except IndexError:
            raise UserWarning('There are less gpus provided than threads available! Please restart with the same number of pipeline processors as GPUs provided and restart! Stopping this thread!')
    else:
        gpu = settings[picking_name]['--gpu']

    command.append('--gpu')
    command.append('{0}'.format(gpu))

    for key, value in settings[picking_name].items():
        if key in ignore_list:
            continue
        elif value:
            try:
                if '|||' in value:
                    external_log, local_key = value.split('|||')
                    with open(settings[external_log], 'r') as read:
                        log_data = json.load(read)
                    try:
                        set_value = log_data[settings['current_set']][picking_name][local_key]['new_file']
                    except KeyError:
                        continue
                else:
                    set_value = value
            except TypeError:
                set_value = value
            command.append(key)
            command.append('{0}'.format(set_value))

    return ' '.join(command), gpu


def create_cryolo_v1_0_4_command(
        picking_name, file_input, file_output, settings, name
        ):
    """Create the crYOLO v1.0.4 command"""

    command = []
    # Start the program
    ignore_list = []
    ignore_list.append('Filter micrographs')
    ignore_list.append('Filter value high pass (A)')
    ignore_list.append('Filter value low pass (A)')
    ignore_list.append('Pixel size (A/px)')
    ignore_list.append('Box size')
    ignore_list.append('Split Gpu?')
    ignore_list.append('--gpu')

    command.append('{0}'.format(settings['Path'][picking_name]))

    command.append('-i')
    command.append('{0}'.format(' '.join(file_input)))
    command.append('-o')
    command.append('{0}'.format(file_output))
    command.append('--write_empty')

    if settings[picking_name]['Split Gpu?'] == 'True':
        try:
            gpu_id = int(name.split('_')[-1])-1
        except ValueError:
            gpu_id = 0
        try:
            gpu = settings[picking_name]['--gpu'].split()[gpu_id]
        except IndexError:
            raise UserWarning('There are less gpus provided than threads available! Please restart with the same number of pipeline processors as GPUs provided and restart! Stopping this thread!')
    else:
        gpu = settings[picking_name]['--gpu']

    command.append('--gpu')
    command.append('{0}'.format(gpu))

    for key, value in settings[picking_name].items():
        if key in ignore_list:
            continue
        elif value:
            try:
                if '|||' in value:
                    external_log, local_key = value.split('|||')
                    with open(settings[external_log], 'r') as read:
                        log_data = json.load(read)
                    try:
                        set_value = log_data[settings['current_set']][picking_name][local_key]['new_file']
                    except KeyError:
                        continue
                else:
                    set_value = value
            except TypeError:
                set_value = value
            command.append(key)
            command.append('{0}'.format(set_value))

    return ' '.join(command), gpu


@tu.rerun_function_in_case_of_error
def create_box_jpg(file_name, settings, queue_com, name):
    """
    Create jpg files that are overlayed with the box coordinate

    file_name - Name of the file to overlay.
    settings - Transphire settings.
    queue_com - Dictionary for communication.
    name - Thread name

    Return:
    It creates a file.
    """
    folder_name = 'picking_folder_feedback_{0}'.format(settings['do_feedback_loop'].value)
    picking_name = settings['Copy']['Picking']
    box_file = [entry for entry in find_logfiles(file_name, file_name, settings, queue_com, name)[0] if entry.endswith('.box') and '/EMAN_START_END/' not in entry][0]
    jpg_file = os.path.join(settings[folder_name], '{0}.png'.format(file_name))
    new_jpg_file = os.path.join(settings[folder_name], 'jpg', '{0}.jpg'.format(file_name))
    tu.mkdir_p(os.path.join(settings[folder_name], 'jpg'))

    box_data = np.atleast_2d(np.genfromtxt(box_file).astype(int))
    jpg_data = imageio.imread(jpg_file, as_gray=False, pilmode='RGB')

    bin_value = 4
    if box_data.size > 0:
        if box_file.endswith('.box'):
            try:
                box_data[:, 0] += box_data[:, 2]//2
                box_data[:, 1] += box_data[:, 3]//2
            except IndexError:
                pass
        elif box_file.endswith('.txt'):
            pass
        else:
            assert 'Box ending not known!', box_file
        box_data[:, 0] = box_data[:, 0]//bin_value
        box_data[:, 1] = box_data[:, 1]//bin_value

        jpg_data = np.rot90(jpg_data, 3)
        create_box(jpg_data=jpg_data, maskcenters=box_data[:,[0,1]], box_size=int(settings[picking_name]['Box size'])//bin_value)
        create_circle(jpg_data=jpg_data, maskcenters=box_data[:,[0,1]], radius=2)
        jpg_data = np.rot90(jpg_data, 1)
    else:
        pass

    imageio.imwrite(new_jpg_file, jpg_data)


def create_box(jpg_data, maskcenters, box_size):
    output_shape = (jpg_data.shape[0],jpg_data.shape[1])
    rect_mask = np.zeros((box_size, box_size))
    for i in range(2):
        rect_mask[:, i] = 1
        rect_mask[:, -i-1] = 1
        rect_mask[i, :] = 1
        rect_mask[-i-1, :] = 1

    idx_mask_x, idx_mask_y = np.where(rect_mask)
    out = np.zeros(output_shape, dtype=bool)

    idx_mask_x_abs = maskcenters[:, None, 0] + idx_mask_x - box_size//2
    idx_mask_y_abs = maskcenters[:, None, 1] + idx_mask_y - box_size//2

    valid_mask = (idx_mask_x_abs >= 0) & (idx_mask_x_abs < output_shape[0]) & \
             (idx_mask_y_abs >= 0) & (idx_mask_y_abs < output_shape[1])

    out[idx_mask_x_abs[valid_mask], idx_mask_y_abs[valid_mask]] = 1
    jpg_data[out, :] = np.array([255, 0, 0])


def create_circle(jpg_data, maskcenters, radius):
    """
    Create a circle in a numpy array

    Arguments:
    jpg_data - Data that needs to be changed
    maskcenters - center of mask
    radius - Radius of the circle

    Returns:
    None, jpg_data will be changed in-place
    """

    output_shape = (jpg_data.shape[0],jpg_data.shape[1])
    X,Y = [np.arange(-radius, radius+1)]*2
    disk_mask = X[:, None]**2 + Y**2 <= radius*radius
    idx_mask_x, idx_mask_y = np.where(disk_mask)

    out = np.zeros(output_shape, dtype=bool)

    idx_mask_x_abs = maskcenters[:, None, 0] + idx_mask_x - radius
    idx_mask_y_abs = maskcenters[:, None, 1] + idx_mask_y - radius

    valid_mask = (idx_mask_x_abs >= 0) & (idx_mask_x_abs < output_shape[0]) & \
             (idx_mask_y_abs >= 0) & (idx_mask_y_abs < output_shape[1])

    out[idx_mask_x_abs[valid_mask], idx_mask_y_abs[valid_mask]] = 1
    jpg_data[out, :] = np.array([255, 0, 0])



"""
TranSPHIRE is supposed to help with the cryo-EM data collection
Copyright (C) 2017 Markus Stabrin

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import glob
import os
from PyQt5.QtWidgets import QHBoxLayout, QWidget, QPushButton, QApplication, QVBoxLayout, QLabel, QComboBox
from PyQt5.QtCore import pyqtSlot, pyqtSignal

from . import transphire_utils as tu

class ButtonContainer(QWidget):
    """
    ButtonContainer widget.

    Inherits from:
    QWidget

    Buttons:
    Save - Save settings
    Load - Load settings
    Default settings - Change default settings
    About - Show about information
    Check quota - Check the quota of current project and scratch directory
    Start/Stop - Start/Stop processing

    Signals:
    sig_load - Emited when the Load Button is clicked (No object)
    sig_save - Emited when the Save Button is clicked (No object)
    sig_start - Emited when the Start Button is clicked (No object)
    sig_stop - Emited when the Stop Button is clicked (No object)
    sig_check_quota - Emited when the Check quota Button is clicked (No object)
    """
    sig_load = pyqtSignal()
    sig_save = pyqtSignal()
    sig_start = pyqtSignal()
    sig_stop = pyqtSignal()
    sig_monitor_start = pyqtSignal()
    sig_monitor_stop = pyqtSignal()
    sig_check_quota = pyqtSignal()

    def __init__(self, parent=None, **kwargs):
        """
        Setup the layout for the widget

        Arguments:
        parent - Parent widget (default None)
        kwargs - Unused arguments for easier automatisation

        Return:
        None
        """
        super(ButtonContainer, self).__init__(parent)

        # Variables
        kwargs = kwargs
        self.settings_folder = kwargs['settings_folder']
        self.template_name = kwargs['template_name']
        self.parent = parent

        # Setup layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        templates = sorted([
            os.path.basename(entry)
            for entry in glob.glob(os.path.join(self.settings_folder, '*'))
            if os.path.isdir(entry)
            ])
        layout_v = QVBoxLayout()
        layout_v.setContentsMargins(0, 0, 0, 0)
        label = QLabel('Chosen template:')
        label.setToolTip('Chosen template')
        layout_v.addWidget(label)

        self.template_box = QComboBox(parent=self)
        self.template_box.clear()
        self.template_box.currentTextChanged.connect(self.change_tooltip)
        self.template_box.addItems([entry for entry in templates if entry != 'SHARED'])
        self.template_box.setCurrentText(self.template_name)
        self.template_box.currentTextChanged.connect(self.select_template)

        layout.addWidget(self.template_box)
        layout_v.addWidget(self.template_box)
        layout.addLayout(layout_v)

        layout_v = QVBoxLayout()
        layout_v.setContentsMargins(0, 0, 0, 0)
        # About button
        self.show_about = QPushButton('About', self)
        self.show_about.setToolTip('About')
        self.show_about.clicked.connect(self._show_about)
        self.show_about.setObjectName('button')
        layout_v.addWidget(self.show_about)

        # Default settings button
        self.default_settings = QPushButton('Default settings', self)
        self.default_settings.setToolTip('Default settings')
        self.default_settings.clicked.connect(self._modify_settings)
        self.default_settings.setObjectName('button')
        layout_v.addWidget(self.default_settings)
        layout.addLayout(layout_v)

        layout_v = QVBoxLayout()
        layout_v.setContentsMargins(0, 0, 0, 0)
        # Load button
        self.load_button = QPushButton('Load', self)
        self.load_button.setToolTip('Load')
        self.load_button.clicked.connect(self.sig_load.emit)
        self.load_button.setObjectName('button')
        layout_v.addWidget(self.load_button)

        # Save button
        self.save_button = QPushButton('Save', self)
        self.save_button.setToolTip('Save')
        self.save_button.clicked.connect(self.sig_save.emit)
        self.save_button.setObjectName('button')
        layout_v.addWidget(self.save_button)
        layout.addLayout(layout_v)

        layout.addStretch(1)

        # Check quota button
        layout_v = QVBoxLayout()
        layout_v.setContentsMargins(0, 0, 0, 0)
        self.check_quota = QPushButton('Check quota', self)
        self.check_quota.setToolTip('Check quota')
        self.check_quota.clicked.connect(self.sig_check_quota.emit)
        self.check_quota.setObjectName('button')
        layout_v.addWidget(self.check_quota)
        layout_v.addWidget(QLabel(self))
        layout.addLayout(layout_v)


        layout_v = QVBoxLayout()
        layout_v.setContentsMargins(0, 0, 0, 0)
        # Start/Stop monitor button
        self.start_monitor_button = QPushButton('Monitor', self)
        self.start_monitor_button.setToolTip('Monitor')
        self.start_monitor_button.clicked.connect(self._start_stop)
        self.start_monitor_button.setObjectName('start')
        self.start_monitor_button.setVisible(True)
        self.start_monitor_button.setEnabled(True)
        layout_v.addWidget(self.start_monitor_button)

        self.stop_monitor_button = QPushButton('Monitor', self)
        self.stop_monitor_button.setToolTip('Monitor')
        self.stop_monitor_button.clicked.connect(self._start_stop)
        self.stop_monitor_button.setObjectName('stop')
        self.stop_monitor_button.setVisible(False)
        self.stop_monitor_button.setEnabled(False)
        layout_v.addWidget(self.stop_monitor_button)

        # Start/Stop button
        self.start_button = QPushButton('Start', self)
        self.start_button.setToolTip('Start')
        self.start_button.clicked.connect(self._start_stop)
        self.start_button.setObjectName('start')
        self.start_button.setVisible(True)
        self.start_button.setEnabled(True)
        layout_v.addWidget(self.start_button)

        self.stop_button = QPushButton('Stop', self)
        self.stop_button.setToolTip('Stop')
        self.stop_button.clicked.connect(self._start_stop)
        self.stop_button.setObjectName('stop')
        self.stop_button.setVisible(False)
        self.stop_button.setEnabled(False)
        layout_v.addWidget(self.stop_button)
        layout.addLayout(layout_v)

        # Final stretch

    @pyqtSlot(str)
    def change_tooltip(self, text):
        self.sender().setToolTip(text)

    @pyqtSlot(str)
    def select_template(self, text):
        self.parent.enable(var=False, use_all=True)
        new_content = {}
        new_design = {}
        for key, value in self.parent.content_raw[text].items():
            if key not in self.parent.content:
                continue
            elif not hasattr(self.parent.content[key], 'set_settings'):
                continue
            new_content[key] = {}
            new_design[key] = {}
            for entry in value[0]:
                for key_entry, value_entry in entry.items():
                    if 'WIDGETS ' in key_entry:
                        new_design[key][key_entry] = value_entry[0]
                        continue
                    new_content[key][key_entry] = value_entry[0]
                    new_design[key][key_entry] = value_entry[1]['widget_2']
        self.parent.set_settings(new_content)
        self.parent.set_design(new_design)
        self.parent.enable(var=True, use_all=True)

    @pyqtSlot()
    def _modify_settings(self):
        """
        Open default settings dialog.
        Called when the Default settings button is clicked.

        Arguments:
        None

        Return:
        None
        """

        temp_save_file = 'temp_reset_save_1'
        content, apply = DefaultSettings.get_content_default(
            edit_settings=True,
            apply=True,
            settings_folder=self.settings_folder,
            template_name=self.template_name,
            )
        template_name = content['DEFAULT']['Others'][0][0]['Default template'][0]
        if not apply:
            tu.message('Restart GUI to apply saved changes')

        elif apply:
            template_name = content['DEFAULT']['Others'][0][0]['Default template'][0]
            self.parent.content_raw = content
            self.parent.content_pipeline = content['DEFAULT']['Pipeline']
            self.parent.enable(var=False, use_all=True)
            # result is True if No is chosen
            result = tu.question(
                head='Keep current values',
                text='Do you want to keep your current settings?\n' +
                'Otherwhise they will be overwritten by the default values.',
                )
            app = QApplication.instance()
            app.setStyleSheet(
                tu.look_and_feel(app=app, default=content['DEFAULT']['Font'])
                )

            if result:
                load_file, _ = self.parent.save(temp_save_file, do_message=False, temp=True, interactive=True)
            else:
                load_file=False
            self.parent.sig_reset.emit(template_name, load_file)

        else:
            pass

    @pyqtSlot()
    def _start_stop(self):
        """
        Start or stop the process dependent on the current button text.
        This method is called when the Start/Stop button is pressed.

        Arguments:
        None

        Return:
        None
        """
        if self.sender().text() == 'Start':
            self.sig_start.emit()
        elif self.sender().text() == 'Stop':
            self.sig_stop.emit()
        elif self.sender().text() == 'Monitor' and self.sender().objectName() == 'start':
            self.sig_monitor_start.emit()
        elif self.sender().text() == 'Monitor' and self.sender().objectName() == 'stop':
            self.sig_monitor_stop.emit()
        else:
            print(
                'Button text not known!',
                'Stopping now, but please contact the TranSPHIRE Authors',
                self.sender().text()
                )
            self.sig_stop.emit()

    def enable(self, var, use_all):
        """
        Enable or disable the buttons.

        Arguments:
        var - State of buttons (True or False)
        use_all - Disable all buttons (True) or only some (False)

        Return:
        None
        """
        assert var is True or var is False, 'Unknown option for var'
        assert use_all is True or use_all is False, 'Unknown option for use_all'
        if use_all:
            self.load_button.setEnabled(var)
            self.save_button.setEnabled(var)
            self.default_settings.setEnabled(var)
            self.start_button.setEnabled(var)
            self.stop_button.setEnabled(var)
            self.start_monitor_button.setEnabled(var)
            self.stop_monitor_button.setEnabled(var)
            self.show_about.setEnabled(var)
            self.check_quota.setEnabled(var)
            self.template_box.setEnabled(var)
        else:
            self.load_button.setEnabled(var)
            self.save_button.setEnabled(var)
            self.check_quota.setEnabled(var)
            self.default_settings.setEnabled(var)
            self.template_box.setEnabled(var)

    @staticmethod
    def _show_about():
        """
        Show the about text.

        Arguments:
        None

        Return:
        None
        """
        text = (
            "\n"
            "TranSPHIRE is supposed to help with the cryo-EM data collection\n"
            "Copyright (C) 2017 Markus Stabrin\n"
            "\n"
            "This program is free software: you can redistribute it and/or modify\n"
            "it under the terms of the GNU General Public License as published by\n"
            "the Free Software Foundation, either version 3 of the License, or\n"
            "(at your option) any later version.\n"
            "\n"
            "This program is distributed in the hope that it will be useful,\n"
            "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
            "GNU General Public License for more details.\n"
            "\n"
            "You should have received a copy of the GNU General Public License\n"
            "along with this program.  If not, see <http://www.gnu.org/licenses/>.\n"
            )
        tu.message(text)

from .loadwindow import DefaultSettings
